;$Id: shapelets_amoeba.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
; ***********************************************************************
; ***********************************************************************
;
; NAME:
;      SHAPELETS_CHI2_CALC
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE: 
;      Computes the chi2 difference between the input and the reconstructed
;      image for a given value of beta (w) and n_max
;
; INPUTS: 
;      pstamp - pstamp structure from shapelets_image2pstamp.pro
;      w    - shapelet scale size beta
;      nmax - maximum order of shapelet decomposition
;
; OPTIONAL INPUTS:
;      None.
;
; KEYWORD PARAMETERS:
;      None.
;
; OUTPUTS: 
;      Reduced chi^2 is returned
;
; OPTIONAL OUTPUTS: 
;      decomp - the shapelet decomposition structure is returned
;      recomp - a reconstructed image of the shapelet model
;
; MODIFICATION HISTORY:
;      Apr 04 - Sky background fitting implemented by RM
;      Apr 02 - PSF deconvolution implemented by Richard Massey
;      Apr 02 - Written by Alexandre Refregier

function shapelets_calc_chi2, pstamp, beta, nmax, x0=x0, DECOMP=d, RECOMP=r, $
                              PSF=psf, PRINTIT=printit, SKY=sky

; Should always have had to specify at least the object's centroid
if not keyword_set(x0) then begin
  x0=size(pstamp.image,/dimensions)/2.
  message,'Need to set centroid!'
endif

; Decompose image into shapelets
shapelets_decomp,pstamp.image,beta,nmax,d,r,x0=x0,/ls,/over,noise=pstamp.noise,$
  /integrate,psf=psf,sky=sky

; Return chi^2 of fit
return,d.chisq[1]

end




; ***********************************************************************
; ***********************************************************************
;
; NAME:
;      SHAPELETS_AMOEBA_CHI2
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE: 
;      Compute Chi^2(beta) in the format required for the amoeba
;      minimisation procudure shapelets_amoeba.pro. This version also 
;      finds the optimal centroid, x0.
;
; INPUTS: 
;      BETA    - Shapelet scale size beta
; 
; COMMON VARIABLES:
;      PSTAMP  - Object structure from shapelets_extract_pstamp.pro
;      X0      - Position of the centre of the shapelet basis functions
;      NMAX    - Value of n_max to use
;      FLAG    - Flag for decomposition
;
; OPTIONAL INPUTS:
;      None.
;
; KEYWORD PARAMETERS:
;      None.
;
; OUTPUTS:
;      Reduced chi^2(beta) is returned.
;      Updates position of the centre of the shapelet basis functions
;      in the common block. Also possibly alters the flag.
;
; MODIFICATION HISTORY:
;      Apr 04 - Sky background fitting implemented by RM
;      Apr 02 - PSF deconvolution implemented by Richard Massey
;      Apr 02 - Written by Alexandre Refregier


function shapelets_amoeba_chi2,beta,printit=printit,psf=psf,sky=sky

common betax0_common,pstamp_1,nmax_1,x0_1,hist,th_min_1,flag

; Compute geometrical constraints
th_max=min([x0_1(0),pstamp_1.im_ran(1)-pstamp_1.im_ran(0)-x0_1(0),$
            x0_1(1),pstamp_1.im_ran(3)-pstamp_1.im_ran(2)-x0_1(1)])

; Make a shapelet decomposition with the new parameters and calculate
;    its chi^2 value.
if (beta le 0.) or beta*sqrt(nmax_1+1) gt th_max or $
     beta/sqrt(nmax_1+1) lt th_min_1 then begin
  c2=1e9                    ; Bounce of walls of allowed n_max vs beta
  xc=x0_1                   ;  space by setting chi^2 to be huge
  flag=(flag>1)
endif else begin
  c2=shapelets_calc_chi2(pstamp_1,beta,nmax_1,x0=x0_1,decomp=d,psf=psf,sky=sky)
  ; Set centre of basis functions for next iteration to the location
  ;   of the current decomposition's first moments.
  xc=shapelets_centroid(d)
endelse

; Update the history records
hist.i=hist.i+1
hist.beta(hist.i)=beta
hist.nmax(hist.i)=nmax_1
hist.x0(*,hist.i)=x0_1
hist.chi2(hist.i)=c2
if keyword_set(printit) then message," n_max, beta, chi^2, x_c:"+$
  string(nmax_1)+string(beta)+string(c2)+string(x0_1[0])+string(x0_1[1]),/info

; Move shapelet centre to current centroid
x0_1=xc

; Set a flag if this moves the outside of the shapelet decomposition
;   off the edge of the postage stamp
npix_x=1+pstamp_1.im_ran(3)-pstamp_1.im_ran(2)
npix_y=1+pstamp_1.im_ran(1)-pstamp_1.im_ran(0)
th_max=beta*sqrt(nmax_1+1)-1 ; -1 for good luck: allow a little leeway!
if min(xc) lt th_max or xc(1) gt npix_x-th_max or xc(1) gt npix_y-th_max then flag=(flag>5)

return,c2

end



; ***********************************************************************
; ***********************************************************************
;
; NAME:
;      AMOTRY1
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Extrapolates by a factor fac through the face of the simplex, across
;      from the high point, tries it and replaces the high point if the new
;      point is better.
;
; MODIFICATION HISTORY:
;      Apr 04 - Sky background fitting implemented by RM
;      Apr 02 - PSF deconvolution implemented by Richard Massey
;      Apr 02 - Adapted from Numerical Recipies by Alexandre Refregier

function amotry1,p,y,func,ilo,ihi,fac,psf=psf,printit=printit,sky=sky

COMPILE_OPT idl2, HIDDEN

ptry=p[ilo]+fac*(p[ihi]-p[ilo])
ytry=call_function(func,ptry,printit=printit,psf=psf,sky=sky)
if ytry lt y[ihi] then begin
  y[ihi]=ytry
  p[ihi]=ptry
endif
return,ytry
end






; ***********************************************************************
; ***********************************************************************
;
;+
; NAME:
;      SHAPELETS_AMOEBA
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Minimise a function f(x). This is a one dimensional version
;      of Numerical Recipies' Amoeba.
;
; INPUTS:
;      ftol            - fractional tolerance to be achieved in the function
;      starting point  - simplex, or p0 and scale, must be given (see below)
;
; OPTIONAL INPUTS:
;      function_name   - a string containing the name of the
;                        function to be minimized (default='func')
;      simplex=[x0,x1] - starting simplex. If not set, scale and p0 
;                        must be used. Returns the ordered simplex if set. 
;                        In particular, simplex(0) corresponds to the 
;                        function minimum
;      p0              - starting point. Must be used with scale.
;      scale           - scale for the starting point. Must be used with p0.
;      function_value  - on exit contains the function values at the
;                        simplex points. The first element contains the 
;                        function minimum. (output only)
;      nmax            - maximum number of calls to func (default=100)
;                        note that ncalls can exceed nmax by a few.
;      ncalls          - returns the number of calls to the function
;
; OUTPUTS:
;      See above.
;
; MODIFICATION HISTORY:
;      Apr 04 - Sky background fitting implemented by RM
;      Apr 02 - PSF deconvolution implemented by Richard Massey
;      Apr 02 - Written by Alexandre Refregier
;-

function shapelets_amoeba,ftol,function_name=func,ncalls=ncalls,nmax=nmax,$
  p0=p0,scale=scale,simplex=simplex,function_value=function_value,$
  printit=printit,flag=flag,psf=psf,sky=sky

; declarations
npts=2           ; number of points in the simplex
if not keyword_set(nmax) then nmax=100  ; maximum number of iterations allowed
if not keyword_set(func) then func='func'

; initialise simplex
if keyword_set(simplex) then begin
  p=simplex
endif else begin
  if not (keyword_set(scale) and keyword_set(p0)) then $
    message,'Simplex or starting point and scale MUST be specified!'
  p=p0+[0.,scale]  ; if not specified set starting simplex using p0 and scale
endelse

; initialise function at simplex points
y=fltarr(npts)
for i=0,npts-1 do y(i)=call_function(func,p(i),printit=printit,psf=psf,sky=sky)
ncalls=npts

; iterate by moving the simplex
while ncalls lt nmax do begin

  ; sort simplex
  ilo=0 & ihi=1
  if y(0) gt y(1) then begin
    ilo=1 & ihi=0
  endif

  ; test for tolerance
  d=total(abs(y))
  if d ne 0 then rtol=abs(y(ihi)-y(ilo))*2./d else rtol=ftol/2.
  if rtol lt ftol then begin  ; done?
    if keyword_set(simplex) then simplex=[p(ilo),p(ihi)]
    if keyword_set(function_value) then function_value=[y(ilo),y(ihi)]
    ;flag=3
    return,p(ilo)
  endif

  ; first reflect about lowest point
  ytry=amotry1(p,y,func,ilo,ihi,-1.,psf=psf,sky=sky)
  ncalls=ncalls+1

  ; if new point lower than lowest point?
  if ytry le y(ilo) then begin
    ytry=amotry1(p,y,func,ilo,ihi,2.,psf=psf,sky=sky)   ; try further expansion if so
    ncalls=ncalls+1
  endif else begin  ; if not, try contraction about lowest point
    ysave=y(ihi)
    ytry=amotry1(p,y,func,ilo,ihi,0.5,psf=psf,sky=sky)
    ncalls=ncalls+1
    if ytry ge ysave then begin   ; if latest point is worse then contract
                                  ; about lowest point
      p(ilo)=p(ilo)-.5*(p(ilo)-p(ihi))
      y(ilo)=ytry
    endif
  endelse

endwhile

; in case the convergence was not achieved with nmax steps
print,'Warning: Amoeba1 failed to converge'
flag=(flag>4)
ilo=0 & ihi=1    ; sort simplex one last time
if y(0) gt y(1) then begin
  ilo=1 & ihi=0
endif
if keyword_set(simplex) then simplex=[p(ilo),p(ihi)]
if keyword_set(function_value) then function_value=[y(ilo),y(ihi)]
return,p(ilo)

end
