;$Id: shapelets_focus.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;       SHAPELETS_FOCUS
;
; PURPOSE:
;       Find the optimal beta, nmax and centroid for a shapelet
;       decomposition via an iterative search throughout the space spanned
;       by these (meta-)variables. See the paper Polar shapelets, by
;       Massey & Refregier (2004) for more details.
;
; CATEGORY:
;       Shapelets.
;
; INPUTS:
;       pstamp - Postage stamp structure containing the image (and other
;                parameters) around a single object. This can be created with
;                shapelets_image2pstamp.pro
;
; OPTIONAL INPUTS:
;       BETA_GUESS - Guess starting point for beta (DEFAULT: guesses itself) 
;       NMAX_GUESS - Guess starting point for nmax (DEFAULT: guesses itself)
;       TOL_CHI2   - Fractional tolerance for Chi^2 minimisation
;       TOL_BETA   - Fractional tolerance for finding beta
;       CHI20      - Target value of reduced Chi^2 to achieve (DEFAULT: 1.0)
;       TH_MIN_GEOM- Minimum geometrical scale theta_min (DEFAULT: 0.5 pixels) 
;       NMAX_RAN   - Range of allowed nmax values (DEFAULT: [2,12])
;       PSF        - A Cartesian decomp structure representing the local PSF.
;                    The shapelet model of the galaxy will be deconvolved from
;                    this.
;
; KEYWORD PARAMETERS:
;       SILENT     - Operate silently.
;       VERBOSE    - Operate noisily.
;       SKY        - 1: fit sky background with a constant value around object.
;                    2: fit sky gradient with a plane around object.
;                    DEFAULT: no sky subtraction.
;
; OUTPUTS:
;       DECOMP     - Decomposition structure containing the shapelet
;                    coefficients with the optimal beta, n_max and centroid
;                    parameters.
;       FOCUS      - IDL structure tracing the search on the beta-nmax plane.
;
; OPTIONAL OUTPUTS:
;       RECOMP     - Recomposed image using optimal shapelet parameters.
;
; CALLING SEQUENCE:
;       shapelets_focus,pstamp,focus,decomp
;
; TO DO:
;       Should check through history list at end to find chisq closest to 1.
;       On occasion, we have gone past this and stopped at something else.
;       Will need to store chi-squareds in history structure to implement this!
;
; NOTES:
;       MEANINGS OF FLAG:
;       0: OK
;       1: has bounced into theta_min or max wall at some point
;       2: nmax_max reached - shapelets incompletely represent object
;       3: chi2>1 but converged (in nmax search) by flatness limit
;       4: did not converge
;       5: centroid off edge
;       6: fatal error
;       NB it can only be one of these. If duplicated, highest number takes
;          priority.
;
; MODIFICATION HISTORY:
;       Jul 04 - Header updated to reflect new additional options by RM.
;       May 04 - Tidied up by RM.
;       Apr 02 - PSF deconvolution implemented by R. Massey.
;       Apr 02 - Written by A. Refregier.
;-

pro shapelets_focus, pstamp, focus, decomp, recomp,                     $
                     PSF=psf, SKY=sky,                                  $
                     BETA_GUESS=beta_guess, NMAX_GUESS=nmax_guess,      $
                     CHI20=chi20, TOL_CHI2=tol_chi2, TOL_BETA=tol_beta, $
                     NMAX_RAN=nmax_ran, TH_MIN_GEOM=th_min_geom,        $
                     VERBOSE=verbose, SILENT=silent


; Initial declarations
if not keyword_set(chi20) then chi20=1.
if not keyword_set(tol_beta) then tol_beta=1e-2
si=size(pstamp.image)
n_data=si(1)*si(2)
chi2_sigma=sqrt(2./n_data)      ; rms for reduced Chi^2
if not keyword_set(tol_chi2) then tol_chi2=chi2_sigma
tol_chi2w=1e-3
if not keyword_set(nmax_ran) then nmax_ran=[2,12]


; Guess startting points if required
if not keyword_set(th_min_geom) then th_min_geom=.5
shapelets_guess_nmaxbeta,pstamp,guess,th_min_geom=th_min_geom*1.05
       ; 1.05 factor present to ensure that guess is not in forbidden area
if not keyword_set(beta_guess) then beta_guess=guess.beta
if not keyword_set(nmax_guess) then nmax_guess=(guess.nmax>nmax_ran(0))<nmax_ran(1)
x0_guess=guess.x0
if keyword_set(verbose) then message,"Guessing n_max, beta, x_c:     "+$
  string(nmax_guess)+string(beta_guess)+"             "+string(x0_guess[0])+string(x0_guess[1]),/info


; Find beta which minimises Chi^2 for the guess nmax
nmax=nmax_guess
beta=beta_guess
x0=x0_guess
shapelets_focus_betax0,pstamp,nmax,focus,decomp,recomp,beta_guess=beta,x0_guess=x0,$
  th_min=th_min_geom,tol=1e-2,psf=psf,printit=verbose,sky=sky
beta=focus.beta
x0=focus.x0


; Alternate nmax-finder and beta-finder until tolerance is reached
found=0
iloop=0
while not found do begin
  nmax_old=nmax
  beta_old=beta
  
  ; Try to find a better nmax
  shapelets_focus_nmax,pstamp,beta,focus,decomp,recomp,x0=x0,chi20=chi20,/append,$
         tol_chi2=tol_chi2,th_min=th_min_geom,nmax_ran=nmax_ran,$
	    psf=psf,printit=verbose,sky=sky
  if focus.flag eq 6 then break    ; about to crash. quit before it gets there
  nmax=focus.nmax
  if nmax_old eq nmax then found=1 ; exit if nmax has not changed  
  if focus.flag eq 3 then found=1  ; exit if we've found a plateau 
       ; this helps low S/N and I THINK its OK about falling off edge, but...

  ; Try to find a better beta
  if not found then begin   
    shapelets_focus_betax0,pstamp,nmax,focus,decomp,recomp,beta_guess=beta,x0_guess=x0,$
       /append,psf=psf,printit=verbose,sky=sky;,th_min=th_min_geom,tol=tol_chi2w
    beta=focus.beta
    x0=focus.x0
    if abs(beta-beta_old) le tol_beta then found=1  ; exit if beta has not changed  
  endif
  if focus.flag eq 5 then found=1  ; if its fallen off edge just quit now

  ; Limit the number of loops
  iloop=iloop+1
  if iloop gt 20 then begin
    focus.flag=(focus.flag>4)
    break
  endif
  
endwhile


; Adjust focus structure
focus.beta_guess=beta_guess
focus.nmax_guess=nmax_guess
focus.x0_guess=x0_guess
focus.tol_chi2=tol_chi2
focus.chi20=chi20
focus.th_min=th_min_geom


; Print result to screen
message,'Flags: ('+strtrim(pstamp.flag,1)+','+strtrim(focus.flag,1)+$
        '), n_max='+strtrim(focus.nmax,1)+', chi^2='+strtrim(focus.chi2,1),/info,noprint=silent


end


