pro shapelets_focus_betax0,pstamp,nmax,focus,decomp,recomp,beta_guess=beta_guess,x0_guess=x0_guess,$
    iter_max=iter_max,tol=tol,append=append,th_min=th_min,$
    psf=psf,printit=printit, sky=sky

;$Id: shapelets_focus_betax0.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;       SHAPELETS_FOCUS_BETAX0
;
; PURPOSE:
;       Find the optimal beta and centroid x0 for the decomposition of an
;       image by minimising Chi^2. This is done using a 1-dimensional Amoeba
;       search. The maximum shapelet order nmax is assumed to be known.
;
; CATEGORY:
;       Shapelets.
;
; INPUTS:
;       PSTAMP     - IDL postage stamp structure extracted from a large image
;                    via shapelets_image2pstamp.pro
;       NMAX       - Maximum shapelet order for decomposition
;
; OPTIONAL INPUTS:
;       X0_GUESS   - Guess for centroid (default:pstamp.x0)
;       BETA_GUESS - Initial guess for beta (default: estimated)
;       ITER_MAX   - Maximum number of iterations (default:30)
;       TOL        - Chi^2 tolerance for Amoeba search (default=1e-3)
;       APPEND     - Append the history file to the current
;                    shapelets_focus structure
;       TH_MIN     - Minimum scale theta_min (default:.5 pixels)
;       FOCUS      - Structure containing the prior history of the search
;
; KEYWORD PARAMETERS:
;       PRINTIT    - Verbose output to screen
;
; OUTPUTS:
;       DECOMP     - Decomposition at optimal beta and x0
;       FOCUS      - Structure containing the history of the search
;
; MODIFICATION HISTORY:
;       May 04     - Sky background fitting added by RM
;       Apr 02     - PSF deconvolution added by R. Massey
;       Mar 02     - Written by A. Refregier
;-


common betax0_common,pstamp_1,nmax_1,x0_1,hist,th_min_1,flag


; declarations
if not keyword_set(iter_max) then iter_max=30
if not keyword_set(tol) then tol=1e-2
n_h=iter_max+5    ; history vector length
if not keyword_set(th_min) then th_min=.5
if not keyword_set(append) then flag=0 else flag=focus.flag
th_min_1=th_min
scale_fact=4.3


; guess beta and x0 if not provided
shapelets_guess_nmaxbeta,pstamp,guess;,/printit
if not keyword_set(beta_guess) then begin
  beta_guess=guess.beta
  message,'Guessing beta of '+strtrim(string(beta_guess),2),/info,noprint=1-printit
endif
if not keyword_set(x0_guess) then begin
  x0_guess=guess.x0
  message,'Guessing centroid of ('+strtrim(string(x0_guess[0]),2)+','+$
          strtrim(string(x0_guess[1]),2)+')',/info,noprint=1-printit
endif 
pstamp_1=pstamp
nmax_1=nmax
x0_1=x0_guess


; set scale for amoeba search
beta_max=min([x0_guess(0),pstamp.im_ran(1)-pstamp.im_ran(0)-x0_guess(0),$ 
            x0_guess(1),pstamp.im_ran(3)-pstamp.im_ran(2)-x0_guess(1)])$  ; th_max
		  /sqrt(nmax+1)
beta_min=(th_min*sqrt(nmax+1))
direction=sign(-2.*beta_guess+beta_max+beta_min) ; But which is it closer to? 
scale=direction*beta_guess/scale_fact            ; Go in opposite direction!


; construct history structure
si=size(shapelets_focus,/st)
if keyword_set(append) and si.type ne 0 then begin
  n_shapelets_focus=n_elements(shapelets_focus.beta_h)
  n_hh=n_shapelets_focus+n_h
  i_h=n_shapelets_focus-1
  hist={i:i_h,beta:fltarr(n_hh),nmax:intarr(n_hh),chi2:fltarr(n_hh),$
    x0:fltarr(2,n_hh)}
  hist.beta(0:n_shapelets_focus-1)=shapelets_focus.beta_h
  hist.nmax(0:n_shapelets_focus-1)=shapelets_focus.nmax_h
  hist.chi2(0:n_shapelets_focus-1)=shapelets_focus.chi2_h
  hist.x0(*,0:n_shapelets_focus-1)=shapelets_focus.x0_h
endif else begin
  hist={i:-1,beta:fltarr(n_h),nmax:intarr(n_h),chi2:fltarr(n_h),$
    x0:fltarr(2,n_h)}
endelse


; find beta and x0 by minimizing chi^2
chi2_min=-1.
beta_min=shapelets_amoeba(tol,function_name='shapelets_amoeba_chi2',printit=printit,$
  nmax=iter_max,p0=beta_guess,scale=scale,function_value=chi2_min,flag=flag,psf=psf,sky=sky)


; make one last decomposition at the best beta and x0
shapelets_decomp,pstamp.image,beta_min,nmax,decomp,recomp,$
  x0=x0_1,/ls,/over,noise=pstamp.noise,/integrate,psf=psf
hist.i=hist.i+1
hist.beta(hist.i)=beta_min
hist.nmax(hist.i)=nmax
hist.x0(*,hist.i)=x0_1
hist.chi2(hist.i)=decomp.chisq(1)
if keyword_set(printit) then message,"n_max, beta, chi^2, x_c:"+$
  string(nmax)+string(beta_min)+string(decomp.chisq[1])+string(x0_1[0])+string(x0_1[1]),/info


; store results and history in structure
focus={beta:beta_min,nmax:nmax,x0:x0_1,chi2:decomp.chisq(1),tol_chi2:tol,$
      th_min:th_min,chi20:-1,beta_guess:beta_guess,nmax_guess:nmax,n_iter:hist.i+1,x0_guess:x0_guess,$
      beta_h:hist.beta(0:hist.i),nmax_h:hist.nmax(0:hist.i),$
      chi2_h:hist.chi2(0:hist.i),x0_h:hist.x0(*,0:hist.i),$
	 flag:flag}


end
