pro shapelets_focus_nmax,pstamp,beta,focus,decomp,recomp,chi20=chi20,x0=x0,$
  nmax_ran=nmax_ran,append=append,tol_chi2=tol,converge=converge,$
  th_min=th_min,psf=psf,printit=printit,sky=sky

;$Id: shapelets_focus_nmax.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;       SHAPELETS_FOCUS_NMAX
;
; PURPOSE:
;       Find the optimal n_max for the decomposition of an image by exploring
;       different values until Chi^2 is equal to a set value Chi2_0.
;       The shapelet scale beta is assumed to be known.
;
; CATEGORY:
;       Shapelets.
;
; INPUTS:
;       PSTAMP   - Object structure from shapelets_extract_pstamp.pro
;       BETA     - Shapelet scale for decomposition
;
; OPTIONAL INPUTS:
;       CHI20    - Value of Chi^2 to achieve (DEFAULT: 1+tol)
;       X0       - Centroid for decomposition (DEFAULT: pstamp.x0)
;       NMAX_RAN - Range of nmax value to explore (DEFAULT: [2,12])
;       TOL_CHI2 - Tolerance for Chi^2 flatness (DEFAULT: 1sigma for Chi^2)
;       TH_MIN   - Minimum scale theta_min (DEFAULT: 0.5 pixels) 
;
; KEYWORD PARAMETERS:
;       APPEND   - Append the history file to the current focus structure
;
; OUTPUTS:
;       DECOMP   - Decomposition at optimal beta and x0
;       FOCUS    - Structure containing the history of the search
;
; OPTIONAL OUTPUTS:
;       converge - Convergence flag
;               (0: found chi2=chi20, 1: flatness limit, 2: nmax_max reached)
;
; MODIFICATION HISTORY:
;       May 04 - Sky background fitting added by RM
;       Apr 02 - PSF deconvolution added by Richard Massey
;       Mar 02 - Written by Alexandre Refregier
;-

; initial declarations
si=size(pstamp.image)
n_data=si(1)*si(2)
chi2_sigma=sqrt(2./n_data)      ; rms for reduced Chi^2
shapelets_guess_nmaxbeta,pstamp,guess
if not keyword_set(x0) then x0=guess.x0
if not keyword_set(tol) then begin
  tol=chi2_sigma
  message,'Using chi^2 tolerance of '+strtrim(string(tol),2),/info,noprint=1-printit
endif
if not keyword_set(chi20) then begin
  chi20=1.+tol
  message,'Trying to achieve chi^2 of '+strtrim(string(chi20),2),/info,noprint=1-printit
endif
if not keyword_set(nmax_ran) then nmax_ran=[2,12]
if not keyword_set(tol) then tol=chi2_sigma
converge=0
if not keyword_set(th_min) then th_min=.5
if not keyword_set(append) then flag=0 else flag=focus.flag


; compute allowed range of nmax given the size of the image and the pixel size
th_max=guess.th_max_geom
nmax_max=round(min([(th_max/beta)^2-1.,(beta/th_min)^2-1.]))
nmax_ran2=[nmax_ran[0],min([nmax_ran(1),nmax_max])>nmax_ran(0)]


; build history structure
si=size(focus,/st)
n_h=nmax_ran2[1]-nmax_ran2[0]+2
if keyword_set(append) and si.type ne 0 then begin
  n_focus=n_elements(focus.beta_h)
  n_hh=n_focus+n_h
  i_h=n_focus-1
  hist={i:i_h,beta:fltarr(n_hh),nmax:intarr(n_hh),chi2:fltarr(n_hh),$
    x0:fltarr(2,n_hh)}
  hist.beta(0:n_focus-1)=focus.beta_h
  hist.nmax(0:n_focus-1)=focus.nmax_h
  hist.chi2(0:n_focus-1)=focus.chi2_h
  hist.x0(*,0:n_focus-1)=focus.x0_h
endif else begin
  hist={i:-1,beta:fltarr(n_h),nmax:intarr(n_h),chi2:fltarr(n_h),$
    x0:fltarr(2,n_h)}
endelse


; try several nmax's starting from the lowest allowed value 
nmax=nmax_ran2[0]-2
good=1
flat=0
out=0
decomp_new=1.
recomp_new=1.
while good and not flat and not out do begin
  if nmax+2 gt nmax_ran2[1] then begin   ; out of range?
    out=1
  endif else begin
    nmax=nmax+2
    decomp_old=decomp_new
    recomp_old=recomp_new
    c2=shapelets_calc_chi2(pstamp,beta,nmax,x0=x0,decomp=decomp_new,recomp=recomp_new,$
       psf=psf,printit=printit,sky=sky)
    if keyword_set(printit) then message,"  n_max, beta, chi^2, x_c:"+$
      string(nmax)+string(beta)+string(c2)+string(x0[0])+string(x0[1]),/info

    ; update history file
    hist.i=hist.i+1
    hist.beta(hist.i)=beta
    hist.nmax(hist.i)=nmax
    hist.x0(*,hist.i)=x0
    hist.chi2(hist.i)=c2

    ; is this a good value?
    if nmax ge nmax_ran2(0)+2 then begin
       if abs(c2-hist.chi2(hist.i-1)) lt tol then begin 
         flat=1
         message,'Chi^2 flatness criterion reached',/info,noprint=1-printit
       endif
    endif 
    if c2 lt chi20 then good=0 
  
  endelse ; out?

endwhile


; decide which of the neighbouring values is optimal
if out then begin    ; did we reach nmax_max?
  if keyword_set(printit) then $
    message,'n_max reached maximum',/info,noprint=1-printit
  converge=2 & flag=(flag>2)
  if (nmax_ran2(1) mod 2) eq (nmax mod 2) then begin  ; are nmax_max and nmax
                                                      ; of same shapelets_reflect?
    nmax_opt=nmax     ; yes: choose last value evaluated
    c2_opt=c2
    decomp=decomp_new
    recomp=recomp_new
  endif else begin   ; no: evaluate nmax+1=nmax_max 
     nmax_old=nmax
     decomp_old=decomp_new
     recomp_old=recomp_new
     nmax=nmax+1
     c2_old=c2
     c2=shapelets_calc_chi2(pstamp,beta,nmax,x0=x0,decomp=decomp_new,recomp=recomp_new,$
	 psf=psf,printit=printit,sky=sky)
     if keyword_set(printit) then message,"  n_max, beta, chi^2, x_c:"+$
       string(nmax)+string(beta)+string(c2)+string(x0[0])+string(x0[1]),/info
     hist.i=hist.i+1
     hist.beta(hist.i)=beta
     hist.nmax(hist.i)=nmax
     hist.x0(*,hist.i)=x0
     hist.chi2(hist.i)=c2
     if c2 lt chi20 then begin  ; is new value better than old one?
       c2_opt=c2_old
       nmax_opt=nmax_old
       decomp=decomp_old
       recomp=recomp_old
     endif else begin
       c2_opt=c2
       nmax_opt=nmax
       decomp=decomp_new
       recomp=recomp_new
     endelse
  endelse
endif else begin    ; not out of range
  if flat and good then begin  ; did we stop because of flatness?
    message,'Chi^2 flatness criterion reached',/info,noprint=1-printit
    converge=1 & flag=(flag>3)
    c2_opt=c2         ; yes: then keep last value
    nmax_opt=nmax
    decomp=decomp_new
    recomp=recomp_new
  endif else begin    ; no then pick best value from 3 nearest neighbors
    nmax_mid=nmax-1
    nmax_a=[nmax-2,nmax-1,nmax]
    recomp_mid=1.
    if nmax-1 lt nmax_ran(0) then begin
      c2_mid=1e9
	 flag=(flag>1)
    endif else $
      c2_mid=shapelets_calc_chi2(pstamp,beta,nmax-1,x0=x0,decomp=decomp_mid,recomp=recomp_mid,$
       psf=psf,printit=printit,sky=sky)
     if keyword_set(printit) then message,"  n_max, beta, chi^2, x_c:"+$
       string(nmax)+string(beta)+string(c2)+string(x0[0])+string(x0[1]),/info
    hist.i=hist.i+1
    hist.beta(hist.i)=beta
    hist.nmax(hist.i)=nmax_mid
    hist.x0(*,hist.i)=x0
    hist.chi2(hist.i)=c2_mid
    c2_a=[hist.chi2(hist.i-2),c2_mid,c2]
    if nmax_a(0) lt nmax_ran(0) then c2_a(0)=1e9
    dc2_opt=min(abs(c2_a-chi20),opt)
    nmax_opt=nmax_a(opt)
    c2_opt=c2_a(opt)
    case opt of
      0: begin
           decomp=decomp_old
           recomp=recomp_old
         end
      1: begin
           decomp=decomp_mid
           recomp=recomp_mid
         end
      2: begin
           decomp=decomp_new
           recomp=recomp_new
         end
    endcase
  endelse   ; flat?
endelse

if decomp.n_max ne nmax_opt then begin
  message,'Error in decomp coefficient assignement',/info,noprint=1-printit
  flag=(flag>6)
endif


; store results and history in structure
focus={beta:beta,nmax:nmax_opt,x0:x0,chi2:c2_opt,chi20:chi20,tol_chi2:tol,$
      th_min:th_min,$
      nmax_guess:nmax_ran2(0),beta_guess:beta,x0_guess:x0,n_iter:hist.i+1,$
      beta_h:hist.beta(0:hist.i),nmax_h:hist.nmax(0:hist.i),$
      chi2_h:hist.chi2(0:hist.i),x0_h:hist.x0(*,0:hist.i),$
	 flag:flag}

return
end




