function shapelets_c2p, cartesian_coeffs, REDUCE=reduce, MATRIX=matrix

;$Id: shapelets_c2p.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SHAPELETS_C2P
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Converts input CARTESIAN shapelet coefficients to their equivalent
;      POLAR shapelet representation.
;
; INPUTS:
;       CARTESIAN_COEFFS - Just the array of coefs as a floating point vector.
;               This way, it can be used upon a set of coefficients in either
;               a decomp or a catalogue environment.
;
; KEYWORD PARAMETERS:
;       REDUCE - Removes degenerate coefficients to keep the number of
;                (complex) polar coefs the same as Cartesian ones, using
;                shapelets_polar_reduce.pro.
;
; OUTPUTS:
;       POLAR_COEFFS - Complex vector containing polar shapelet coefficients,
;                      truncated to same n_max as original Cartesian shapelet
;                      coefficients.
;       MATRIX - Optionally returns conversion matrix to speed up multiple 
;                conversions. This matrix is independent of beta etc, so once
;                it has been calculated within the current scope, simply use:
;                polar_coeffs=matrix#cartesian_coeffs
;
; EXAMPLE USE:
;      result=shapelets_c2p([1.0,0.0,0.0,0.5,0.1,0.2])
;
; NOTES:
;      A vector of polar shapelet coefficients may be indexed using:
;        shapelets_make_nvec,n_max,n_r,n_l,n_coeffs
;        n=n_r+n_l
;        m=n_r-n_l
;
; MODIFICATION HISTORY:
;      Richard Massey - February 2002
;-

; Check vector is in the correct orientation for matrix operation to work
;arrsize=size(cartesian_coeffs) & while arrsize(0) gt 1 do reform(cartesian_coeffs)
cartesian_coeffs=reform(cartesian_coeffs)

; Work out what n_max was and set up variables
n_coeffs = n_elements(cartesian_coeffs)
n_max=fix(0.5*(sqrt(8.*n_coeffs+1.)-1.))-1
shapelets_make_nvec, n_max, n1, n2, n_n   
; NB: n_n had better be the same as n_coeffs!
; We could abort here if it is not, because that would indicates there was
; a problem with the input coefficient vector. Or add zero padding.

; Final answer will go here
polar_coeffs = complexarr(n_coeffs)

; Create conversion matrix
if not keyword_set(matrix) then begin
  matrix=complexarr(n_coeffs,n_coeffs)
  
  ;if not keyword_set(calculate) and n_max lt 15 then begin
  ;  message,'Look-up table stored on disc at...'
  ;endif else begin
    for i=0,n_coeffs-1 do begin    ; Cartesian
      for j=0,n_coeffs-1 do begin  ; Polar

        ; Get into form of eqn(69) in Paper I
        n_r=n1(j)
        n_l=n2(j)
        n_1=n1(i)
        n_2=n2(i)

        if n_1+n_2 eq n_r+n_l then begin

          factor=complex(0.,0.)
          for n_rd=0,n_r do begin
            for n_ld=0,n_l do begin
              if n_rd+n_ld eq n_1 then begin
                factor = factor                $
                  + complex(0.,1.)^(n_ld-n_rd) $
                  * comb(n_r,n_rd)             $
                  * comb(n_l,n_ld)
              endif
            endfor
          endfor

          matrix(j,i) =               $
              2.^(-0.5*(n_r+n_l))     $
            * complex(0,1.)^(n_r-n_l)  $
            * sqrt( factorial(n_1)    $
                 *  factorial(n_2)    $
                 /  factorial(n_r)    $
                 /  factorial(n_l)  ) $
            * factor


        endif

        ;if i ge 6 and i le 9 and j ge 6 and j le 9 then print,n_r+n_l,n_r-n_l,n_1,n_2,matrix(j,i)

      endfor
    endfor  
  ;endelse
endif


; Do conversion
polar_coeffs=matrix#cartesian_coeffs

; Remove degenerate coefficients
if keyword_set(reduce) then polar_coeffs=shapelets_polar_reduce(polar_coeffs)

return,polar_coeffs

end
