function shapelets_chi, nm, x1, x2,          $
                        INTEGRATE=integrate, $
                        ARRAY=array,         $
                        BETA_IN=beta_in,     $
                        MATRIX=matrix

;$Id: shapelets_phi.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SHAPELTS_CHI
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Compute (dimensionless) 2D polar shapelet basis functions chi_n_m(x,y),
;      based on Laguerre polynomials. Dimensional basis functions
;      can be calculated with shapelets_chi(nm,x1/beta[,x2/beta])/beta.
;
; INPUTS:
;      nm - Basis function order. Integer vector [n,m].
;
; OPTIONAL INPUTS:
;      x1   - Grid of x coordinates (e.g. created by shapelets_make_xarr.pro).
;      x2   - Grid of y coordinates.
;      beta - Shapalet scale size beta       a single pixel and integrating...
;
; KEYWORD PARAMETERS:
;      /INTEGRATE - Default behaviour is to simply evaluate phi at the centre
;                   of each pixel. If /INTEGRATE is set, the routine will use
;                   the recursion relation in Shapelets III to integrate the
;                   the Cartesian basis functions within pixels, then convert
;                   these to polar shapelets via the matrix in Shapelets I.
;      /ARRAY     - Instead of just returning one basis function, if /AARRAY is
;                   set, the routine will return an array containing all of
;                   the (pixellated) basis functions up to n_max=n.
;
; OUTPUTS:
;      Polar shapelet basis function chi_n1[_n2](x[,y]), in a complex array.
;
; MODIFICATION HISTORY:
;      Oct 03 - Written by R. Massey.
;-

COMPILE_OPT idl2

if n_elements(nm) ne 2 then message,'Polar basis functions can only be calculated in 2D'
if nm[0] lt 0 then message,'n must be non-negative!'
if ((nm[0]-nm[1]) mod 2) then message,'n and m must both be odd or both be even!'

; Set defaults
if keyword_set(beta_in) then beta=beta_in else beta=1
if not keyword_set(x1) or not keyword_set(x2) then begin
  shapelets_make_xarr,[128,128],x1,x2
  x1=x1/8.
  x2=x2/8.
endif
n_pix_x=(size(x1,/DIMENSIONS))[0]
n_pix_y=(size(x1,/DIMENSIONS))[1]

if not keyword_set(integrate) then begin

  ; Form polar co-ordinate grid.
  rsq    = (x1^2+x2^2)/beta^2
  theta  = atan(x2,x1)
  rootpi = sqrt(!pi)
  
  ; Prepare vectors of n, m, n_l and n_r
  if not keyword_set(array) then begin
    n_coeffs=1
    n=nm[0]
    m=nm[1]
  endif else begin
    shapelets_make_nvec,nm[0],nr,nl,n_coeffs
    n=nl+nr
    m=nr-nl
  endelse
  nl=(n-abs(m))/2
  nr=(n+abs(m))/2

  ; Draw basis functions
  const  = ((-1)^nl) / beta * sqrt(factorial(nl)/factorial(nr)) / rootpi
  gauss  = exp(rsq/(-2.))
  rotate = exp(complex(0,!pi/2))
  BasisF = complexarr(n_pix_x,n_pix_y,n_elements(n))
  for i=0,n_coeffs-1 do begin
    shape         = rsq^(abs(m[i]/2.)) * laguerre(rsq,nl[i],abs(m[i])) * $
                    exp(complex(0,m[i]*(theta)))
    BasisF[*,*,i] = const[i] * gauss * shape ;* rotate
  endfor

endif else begin

  ; Integrate phi_n(x,y) within pixels...
  BasisC=shapelets_phi([nm[0],0],x1,x2,/ARRAY,/INTEGRATE)
  shapelets_make_nvec,nm[0],nl,nr,n_coeffs
  BasisC=reform(BasisC,n_pix_x*n_pix_y,n_coeffs)

  ; ...then convert to chi_nm(x,y)
  matrix=0 & a=shapelets_c2p(fltarr(n_coeffs),matrix=matrix)
  if not keyword_set(array) then begin
    BasisF=complexarr(n_pix_x,n_pix_y)
    n=nl+nr
    m=nr-nl
    sel=where(n eq nm[0] and m eq nm[1])
   ;BasisF=transpose(matrix[sel,*] # transpose(BasisC))
    BasisF=reform(matrix[sel,*]) ## BasisC ; faster equivalent
    BasisF=reform(BasisF,n_pix_x,n_pix_y)
  endif else begin
   ;BasisF=transpose(matrix # transpose(BasisC))
    BasisF=transpose(matrix) ## BasisC ; faster equivalent
    BasisF=reform(BasisF,n_pix_x,n_pix_y,n_coeffs)
  endelse

endelse

return, BasisF

end
