pro shapelets_read_image,image,filename,$
    SILENT=silent,FULLPATH=fullpath

;$Id: shapelets_read_image.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;         SHAPELETS_READ_IMAGE
;
; PURPOSE:
;         Read in fits image, plus segmentation and inverse variance pixel
;         map if available (it looks for the same filename, but with
;         _objects and _weight added before the extension).
;
; CATEGORY:
;         Data I/O.
;
; INPUTS:
;         filename - Name of .fits image file.
;
; KEYWORD PARAMETERS:
;          /SILENT - Operates silently.
;        /FULLPATH - If this keyword is set, the routine expects "filename"
;                    to contain the full path and extension of the image file.
;                    Without it, the default is to look in the directory
;                    set in shapelets_paths.pro.
;
; OUTPUTS:
;         Returns an image structure, containing image, header, noise map, etc.
;
; EXAMPLE USE:
;         shapelets_read_image,image,"example"
;
; MODIFICATION HISTORY:
;         Jul 04 - Segmentation maps changed from _objects.fits to
;                  _seg.fits by AR
;         May 04 - Weight maps changed from _noise.fits to _weight.fits by RM
;         Mar 04 - Modified by AR to clarify noise map
;         Apr 02 - Generalised to accept different filenames by R. Massey
;         Mar 02 - Written by A. Refregier (adapted from RM's shex.pro)
;-

;
; Decide file name to look for
;
if keyword_set(filename) then begin
  file=filename ; Don't disrupt input variables
endif else message,"You must specify an image file name!"
if keyword_set(fullpath) then begin
  ; Strip file extension
  file=strmid(file,0,strpos(strlowcase(file),".fit"))
endif else begin
  ; Add relevant path name
  file=shapelets_paths(3,silent=silent)+file
endelse
if file_test(file+'.fits') then begin
  message,'Reading in image data from '+file+'.fits',/info,noprint=silent
endif else message,"Cannot find image "+file+".fits!"


;
; Read fits image
;
fits_read,file+'.fits',image,header


;
; Determine key image properties
;
n_pixels=size(image,/dimensions)
pixel_scale_line=where(strmid(header[*],0,9) eq "PIXSIZE =")
if pixel_scale_line[0] eq -1 then pixel_scale=1. $
else pixel_scale=float(strmid(header[pixel_scale_line],9,22))


;
; Read in or construct noise map
;
if file_test(file+'_weight.fits') then begin
	message,'Reading in inverse variance map',/info,noprint=silent
	fits_read,file+'_weight.fits',noise
endif else begin
     message,'No noise map available',/info,noprint=silent
     noise=0B
;	message,'Manufacturing artificial noise data',/info,noprint=silent
;    background=2.5*median(image)-1.5*mean(image)
;	noise=fltarr(n_pixels)
;	noise_val=1/(mean(abs(image-background)))^2
;	noise(*,*)=noise_val
endelse


;
; Read in "segmented image" created by SExtractor
; (this flags every pixel with the SEx ID of the object it belongs to -
;  or zero, if it is part of the background.)
;
if file_test(file+'_seg.fits') then begin
  message,'Reading in segmented pixel map',/info,noprint=silent
  fits_read,file+'_seg.fits',seg
endif else begin
  message,'Segmented SExtractor pixel map does not exist!',/info,$
        noprint=silent
  seg=0B;intarr(n_pixels)
endelse


;
; Store in image-type IDL structure
;
image={name:filename,           $
       type:"image",            $
	  npix:n_pixels,           $
	  image:image,             $
	  noise:noise,             $
 	  seg:seg,                 $
       pixel_scale:pixel_scale, $
	  header:header}

;
; Following lines added by AR, but noise/seg keywords don't exist so RM commented them out
;
;if keyword_set(noise) then image=create_struct(image,'noise',noise)
;if keyword_set(seg) then image=create_struct(image,'seg',seg)


;
; Write our success story
;
message,'Image '+file+'.fits now in memory',/info,noprint=silent

end
