;$Id: shapelets_read_shapecat.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
; ************************************************************************
; ************************************************************************
;
; NAME:
;       SHAPELETS_SHAPECAT_MOMENTS
;
; PURPOSE:
;       Calculates the basic moments of galaxies in a shapelet catalgoue,
;       and adds those values to the IDL shapecat structure.
;
; CATEGORY:
;       Shapelet catalogue manipulation.
;
; INPUTS:
;       shapecat - IDL shapecat structure.
;
; OUTPUTS:
;       shapecat - IDL shapecat structure, including additional tags.
;
; MODIFICATION HISTORY:
;       May 04 - Written by Richard Massey

pro shapelets_shapecat_moments, shapecat, SILENT=silent

COMPILE_OPT idl2

; Strip catalogue of any old variables to avoid name conflicts and ensure backward compatability
message,'Calculating object moments',/info,noprint=silent
temp_shapecat={name:shapecat.name,type:shapecat.type}
for i=0,n_tags(shapecat)-1 do begin
  n=strupcase((tag_names(shapecat))[i])
  if n ne "NAME" and n ne "TYPE" and n ne "FLUX" and n ne "FLUX_ERROR" $
    and n ne "SNR" and n ne "MAG" and n ne "CENTROID" and n ne "CENTROID_ERROR" $
    and n ne "RSQUARED" and n ne "RSQUARED_ERROR" and n ne "CENTROID" $
    and n ne "CENTROID_ERROR" and n ne "ELLIPTICITY" and n ne "ELLIPTICITY_ERROR" $
    and n ne "QUADRUPOLE" and n ne "QUADRUPOLE_ERROR" and n ne "MOMENTS" then $
    temp_shapecat=create_struct(temp_shapecat,(tag_names(shapecat))[i],shapecat.(i))
endfor
shapecat=temp_shapecat

; Create empty arrays to contain shape moments
shapelets_make_nvec, shapecat.maxn_max, n1, n2, n_coeffs
flux              = fltarr(shapecat.n)
flux_error        = fltarr(shapecat.n)
snr               = fltarr(shapecat.n)
mag               = fltarr(shapecat.n)
centroid          = fltarr(shapecat.n,2)
centroid_error    = fltarr(shapecat.n,2)
rsquared          = fltarr(shapecat.n)
rsquared_error    = fltarr(shapecat.n)
quadrupole        = fltarr(shapecat.n,2,2)
quadrupole_error  = fltarr(shapecat.n,2,2)
ellipticity       = fltarr(shapecat.n,2)
ellipticity_error = fltarr(shapecat.n,2)

; Loop over each object in the catalogue
for i=0,shapecat.n-1 do begin
  ; Extract a (temporary) decomp structure
  decomptemp = {coeffs:reform(shapecat.coeffs[i,*]),                      $
                coeffs_error:reform(shapecat.coeffs_error[i,*]),          $
                n_coeffs:n_coeffs, n1:n1, n2:n2, n_max:shapecat.n_max[i], $
                beta:shapecat.beta[i], x:reform(shapecat.x[i,*]) }
  ; Calculate desired object moments
  temp_centroid = shapelets_centroid(decomptemp,/cartesian,/error)
  Q = shapelets_quadrupole(decomptemp,/cartesian,/error,flux=temp_flux,   $
                rsquared=temp_rsquared,ellipticity=temp_ellipticity)
  ; Store moments in global variables    
  flux[i]                 = temp_flux[0]
  flux_error[i]           = temp_flux[1]
  snr[i]                  = flux[i]/flux_error[i]
  mag[i]                  = -2.5*alog10(flux[i])
  centroid[i,*]           = temp_centroid[*,0]
  centroid_error[i,*]     = temp_centroid[*,1]
  rsquared[i]             = temp_rsquared[0]
  rsquared_error[i]       = temp_rsquared[1]
  quadrupole[i,*,*]       = Q[*,*,0]
  quadrupole_error[i,*,*] = Q[*,*,1]
  ellipticity[i,*]        = [float(temp_ellipticity[0]),imaginary(temp_ellipticity[0])]
  ellipticity_error[i,*]  = [float(temp_ellipticity[1]),imaginary(temp_ellipticity[1])]
endfor

; Add variables containing moments to shapelet catalogue
shapecat=create_struct(shapecat,"moments",1B,                             $
         "flux",flux,"flux_error",flux_error,                             $
         "snr",snr,"mag",mag,                                             $
         "centroid",centroid,"centroid_error",centroid_error,             $
         "rsquared",rsquared,"rsquared_error",rsquared_error,             $
         "quadrupole",quadrupole,"quadrupole_error",quadrupole_error,     $
         "ellipticity",ellipticity,"ellipticity_error",ellipticity_error)

end


; ************************************************************************
; ************************************************************************
;
; NAME:
;      SHAPELETS_SHAPECAT_NMAX
;
; PURPOSE:
;      Alter n_max in a shapelet catalogue.
;
;
; CATEGORY:
;       Shapelet catalogue manipulation.
;
; INPUTS:
;       shapecat - IDL shapecat structure.
;       n_max    - Desired new n_max (can be the same as, less than or
;                  greater than that already used in shapecat).
;
; OUTPUTS:
;       shapecat - IDL shapecat structure.
;
; MODIFICATION HISTORY:
;       May 04 - Written by Richard Massey

pro shapelets_shapecat_nmax, shapecat, n_max, SILENT=silent

COMPILE_OPT idl2

; Do we need to change anything after all?
if (n_max ne shapecat.maxn_max) then begin

  ; Determine what we want n_max to be
  shapelets_make_nvec, n_max, n1, n2, n_coeffs
  
  ; Truncate coefficients to reduce n_max
  if n_max lt shapecat.maxn_max then begin
    message,'Truncating coefficients above '+$
            'n_max='+strtrim(string(n_max),2),/info,noprint=silent
    coeffs=shapecat.coeffs[*,0:n_coeffs-1]
    coeffs_error=shapecat.coeffs_error[*,0:n_coeffs-1]
  endif else begin
  
  ; Pad with zeros to increase n_max
    message,'Padding coefficients with zeros to increase from '+$
            'n_max='+strtrim(string(shapecat.maxn_max),2)+$
            ' to n_max='+strtrim(string(n_max),2),/info,noprint=silent
    coeffs=[[shapecat.coeffs],[fltarr(shapecat.n,n_coeffs-shapecat.n_coeffs)]]
    coeffs_error=[[shapecat.coeffs_error],[fltarr(shapecat.n,n_coeffs-shapecat.n_coeffs)]]
  endelse
  
  ; Adjust shapecat struture to reflect new length of coefficient list
  shapecat_tagnames=tag_names(shapecat)
  temp_shapecat={name:shapecat.name,type:shapecat.type}
  for i=0,n_tags(shapecat)-1 do begin
    case strupcase(shapecat_tagnames[i]) of
      "NAME":
      "TYPE":
      "COEFFS": temp_shapecat=create_struct(temp_shapecat,"coeffs",coeffs)
      "COEFFS_ERROR": temp_shapecat=create_struct(temp_shapecat,"coeffs_error",coeffs_error)
      else: temp_shapecat=create_struct(temp_shapecat,shapecat_tagnames[i],shapecat.(i))
    endcase
  endfor
  shapecat=temp_shapecat
  shapecat.maxn_max=n_max
  shapecat.n_coeffs=n_coeffs
  
endif

end


; ************************************************************************
; ************************************************************************
;
;+
; NAME:
;      SHAPELETS_READ_SHAPECAT
;
; PURPOSE:
;      Reads in a shapelet coefficient catalogue of objects, created by
;      shex.pro, then stores them in a shapecat IDL structure. This may
;      then be converted into decomp structures using 
;      shapelets_shapecat2decomp.pro.
;
; CATEGORY:
;      Data I/O.
;
; INPUTS:
;      shapecat - name of structure where data will be stored.
;      filename - filename to be read, without path or extension.
;
; OPTIONAL INPUTS
;      n_max    - truncates all objects to only n_max coefficients.
;
; KEYWORD PARAMETERS:
;      SILENT   - operates silently.
;      CARTESIAN- converts all objects to Cartesian shapelet coefs on load.
;      POLAR    - converts all objects to polar shapelet coefs on load.
;      MOMENTS  - calculates shapelet unweighted quadrupole moments on load.
;                 This also gives flux and size.
;      PARITY   - reorientates to horizontal (based on phase{a_22}) and
;                  flips parity of those with right(?)-handedness (based on 
;                  phase{a_42}-phase{a_22}) on load.
;
; OUTPUTS:
;      shapecat - Shape information is stored in this structure.
;
; MEANING OF FIRST FLAG (from shapelets_image2obj.pro):
;      0: OK
;      1: did not normalise noise locally (is object filling postage stamp?)
;      2: nearby object (beware overlapping isophotes!)
;      3: both of above
;      4: near edge or a saturated star
;      5: HUGE object (SExtractor gaves whole image occasionally)
;      NB can only be one of these. If duplicated, a higher number takes priority.
;
; MEANINGS OF SECOND FLAG (from shapelets_focus.pro):
;      0: OK
;      1: has bounced into theta_min or max wall at some point
;      2: nmax_max reached - shapelets incompletely represent object
;      3: chi2>1 but converged (in n_max search) by flatness limit - still incomplete
;      4: did not converge
;      5: centroid off edge
;      6: fatal error
;      NB it can only be one of these. If duplicated, highest number has priority.
;
; MODIFICATION HISTORY:
;      May 04 - Written by R.Masey
;-

pro shapelets_read_shapecat, shapecat, filename, silent=silent, $
                  n_max=n_max, polar=polar, cartesian=cartesian,$
                  moments=moments, parity=parity

COMPILE_OPT idl2

; Set defaults
if not keyword_set(polar) then polar=0B
if keyword_set(cartesian) then polar=0B
if keyword_set(resize) then moments=1B
if not keyword_set(filename) then filename='HDF-N_f814'

; Read in catalogue
message,'Reading in catalogue '+filename+'.shapecat',/info,noprint=silent
restore,shapelets_paths(2,/silent)+filename+'.shapecat'

; Adjust n_max of catalogue if desired
if keyword_set(n_max) then shapelets_shapecat_nmax, shapecat, n_max, silent=silent

; Calculate shape moments if desired
if keyword_set(moments) then begin
  ; Check not already calculated
  already_calculated=0
  if moments eq 1 then begin
    shapecat_tagnames=tag_names(shapecat)
    for i=0,n_tags(shapecat)-1 do begin
      if strupcase(shapecat_tagnames[i] eq "MOMENTS") then begin
        if (shapecat.(i) eq 1) then already_calculated=1
      endif
    endfor
  endif
  if already_calculated eq 1 then begin
    message,'Object moments already calculated in save file',/info,noprint=silent
  endif else shapelets_shapecat_moments, shapecat, silent=silent
endif

; Convert to polar coefficients if desired
if keyword_set(polar) then begin
  if shapecat.polar eq 0 then begin
    message,'Converting coefficients to polar shapelets',/info,noprint=silent
    shapecat.polar=1B
    for i=0,shapecat.n-1 do begin
      temp_coeffs=shapelets_c2p(reform(shapecat.coeffs[i,*]),/REDUCE,matrix=matrix)
      shapecat.coeffs[i,*]=transpose(temp_coeffs)
      temp_coeffs_error=shapelets_c2p(reform(shapecat.coeffs_error[i,*]),/REDUCE,matrix=matrix)
      shapecat.coeffs_error[i,*]=transpose(temp_coeffs_error)
    endfor
  endif else message,'Coefficients already in polar shapelet form',/info,noprint=silent
endif else if shapecat.polar eq 1 then begin
  message,'Need to convert coefficients back to Cartesian shapelets!'
endif

; Flip and rotate objects so that they all have the same parity and orentation
if keyword_set(parity) then shapelets_shapecat_reorient, shapecat

; Add in tags for forward compatibility
if not tag_exist(shapecat,"shear_estimates") then shapecat=create_struct(shapecat,"shear_estimates",0B)
if not tag_exist(shapecat,"morphology") then shapecat=create_struct(shapecat,"morphology",0B)

end



