;$Id: shapelets_sexcat2pstamp.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;       SHAPELETS_SEXCAT2PSTAMP
;
; PURPOSE:
;       Extract a postage stamp of image data around an object detected by
;       SExtractor data. Uses the fact that the inverse variance
;       is zero on saturated stars and around the edge of the HDF.
;
; CATEGORY:
;       Shapelets/ROI extraction.
;
; INPUTS:
;       IMAGE  - Image structure (read in with shapelets_read_image.pro)
;       SEXCAT - SExtractor catalogue of image 
;                (read in with shapelets_read_sexcat.pro)
;       ID     - ID number of the object of interest in the SEx catalogue
;
; OPTIONAL INPUTS:
;       NFWHM  - size of pstamp in units of a_sextractor (default=3)
;
; KEYWORD PARAMETERS:
;       NOISE_MAP - Use noise map from image structure (rather than local
;                   background estimation).
;       SEG_MAP   - Use locally-determined segmentation map (rather than the
;                   one from SExtractor, stored in the image structure).
;       CIRCULAR  - Chop off corners to consider a circular region as
;                   opposed to a square.
;       BORDER    - Increase border around the edges of postage stamp, to
;                   make pretty plots in papers but for useless elsewhere.
;       PLOTIT    - Plot the postage stamp to the screen.
;       SILENT    - Operate silently.
;
; OUTPUTS:
;       PSTAMP - Postage stamp structure
;
; MEANINGS OF FLAG: (obects with flags greater than 3 should be discarded)
;       0 - OK
;       1 - (not used)
;       2 - nearby object in postage stamp (but far enough to be isolated)
;       3 - severe overlapping with nearby object
;       4 - near an edge 
;       5 - zero Sextractor FWHM; is this just some noise?
;       6 - too few background pixels around object; huge object or neighbour?
;       7 - object entirely masked out
;
; MODIFICATION HISTORY:
;       Jul 04 - Default behaviour of /seg_map flipped by RM.
;       Jul 04 - sexid field added to the pstamp structure by AR.
;       May 04 - Flags improved by J. Berge to be sure to keep the largest one.
;       Mar 04 - Simplified by A. Refregier to avoid using external noise
;                and segmentation maps; also added circular postage stamp option
;       Dec 02 - Border flag added by RM to make nice plots in papers.
;       Apr 02 - pstamp{}, image{} and sexcat{} structures formalism
;                incorporated by RM
;       Dec 01 - Written by R. Massey.
;-

function shapelets_sexcat2pstamp,image,sexcat,id,           $
    NOISE_MAP=noise_map ,SEG_MAP=seg_map, CIRCULAR=circular,$
    BORDER=border, NFWHM=nfwhm,$             ; add an extra border to look pretty in plots
    SILENT=silent, PLOTIT=plotit             ; verbose or not

COMPILE_OPT idl2

; Initialise default parameters
flag=0
nbgpix_min=10        ; minimum number of pixels required to evaluate background
if not keyword_set(nfwhm) then nfwhm = 2.75
nfwhm = nfwhm + keyword_set(border)   ; Size of postage stamp in unit of Sextractor FWHM
mask_neigh = 2.75                     ; Size of region ot excise from neighbours in units of Sextractor a and b
r_pstamp=round(nfwhm*sexcat.a[id]+4.) ; Postage stamp radius [pixels]
on_error,2 & pstamp={flag:7}          ; If routine crashes, return failure flag

; Guess at bounds of postage stamp using SExtractor information
xmin  = (fix(sexcat.x[id,0])-r_pstamp > 0)< (image.npix[0]-1)
xmax  = (fix(sexcat.x[id,0])+r_pstamp < (image.npix[0]-1) )>0
ymin  = (fix(sexcat.x[id,1])-r_pstamp > 0)< (image.npix[1]-1)
ymax  = (fix(sexcat.x[id,1])+r_pstamp < (image.npix[1]-1))>0


;Some objects in the HDF are just noise around the edges. SExtractor gives
; these zero FWHM
if sexcat.fwhm[id] eq 0. then begin ;RM

  flag=[flag,5]
  message,'This object has zero Sextractor FWHM! Noise?',/info,noprint=silent
endif

; Check if the object is near an edge
if xmin eq 0. or ymin eq 0. or xmax eq (image.npix[0]-1) or $
   ymax eq (image.npix[1]-1) then begin
  flag=[flag,4]
  message,'Object near an edge!',/info,noprint=silent
  return,pstamp
endif


; Extract region of interest
bit   = image.image[xmin:xmax,ymin:ymax]
n_bit = [xmax-xmin+1,ymax-ymin+1]


; Make postage stamp circular by masking off pixels in corners
if keyword_set(circular) then begin
  shapelets_make_xarr,n_bit,x1_c,x2_c,x0=sexcat.x[id,*]-[xmin,ymin] ; construct x-y map centered on the object
  corner=where(round(sqrt(x1_c^2+x2_c^2)-r_pstamp) gt 0,n_corner)
  if n_corner gt 0 then bit[corner]=0.
endif


; Construct segmented pixel map if requested
if (not keyword_set(seg_map)) and (n_elements(image.seg[0]) gt 1)  then begin
  bits=image.seg[xmin:xmax,ymin:ymax]
  if keyword_set(circular) then begin
    if n_corner gt 0 then bits[corner]=0.
  endif
endif else begin
  message,'Local construction of segmentation map',/info,noprint=silent
  bits=fltarr(n_bit[0],n_bit[1])   ; segmentation map
  objsize=mask_neigh*sexcat.a
  if keyword_set(circular) then begin
    message,'Warning: segmentation map for circular postage stamp not fully tested!',/info,noprint=silent
    neigh=where(sexcat.id ne id and $
         (sexcat.x[*,0]-sexcat.x[id,0])^2+(sexcat.x[*,1]-sexcat.x[id,1])^2 lt r_pstamp+objsize,n_neigh)
  endif else begin
    neigh=where(sexcat.id ne id and sexcat.x[*,0]+objsize ge xmin and sexcat.x[*,0]-objsize le xmax and $
         sexcat.x[*,1]+objsize ge ymin and sexcat.x[*,1]-objsize le ymax, n_neigh)
  endelse
  ;if n_neigh gt 1 then begin
  ;  message,'Found'+string(n_neigh,format='(i2)')+' potential neighbours',/info,noprint=silent
  ;endif
  if n_neigh lt 1 then objin=[id] else objin=[neigh,id]      ; all objects in the postage stamp
  n_objin=n_elements(objin)     ; number of neighbouring objects including object of interest
  shapelets_make_xarr,n_bit,x1,x2,x0=[.5,.5]   ; construct x-y map
  x1=x1+xmin & x2=x2+ymin
  for i=0,n_objin-1 do begin
    a=sexcat.a[objin[i]] & b=sexcat.b[objin[i]]
    theta=sexcat.theta[objin[i]]/!radeg
    dx1=x1-sexcat.x[objin[i],0] & dx2=x2-sexcat.x[objin[i],1]
    r2_ellip=(dx1*cos(theta)+dx2*sin(theta))^2/a^2+(-dx1*sin(theta)+dx2*cos(theta))^2/b^2
    gd=where(r2_ellip lt mask_neigh^2,n_gd)
    if objin[i] eq id and n_gd gt 0 then begin   ; check whether there is severe overlapping
      if max(bits[gd]) gt 0 then begin
        flag=[flag,3]
        message,'Severe overlapping with nearby object',/info,noprint=silent
      endif
    endif
    if n_gd gt 0 then bits[gd]=objin[i]+1    ; following Sextractor conventsion
  endfor
endelse
if keyword_set(circular) then bits[corner]=-1


; Check segmented pixel map for any regular objects nearby
othpix= where((bits ne 0) and (bits ne (id+1)) and (bits ne -1),nothpix)  ; pixels in other sources
obpix = where((bits eq (id+1)),nobpix)  
;print,'nothpix',nothpix
;print,'nobpix',nobpix                 
if nobpix eq 0 then begin
  flag=[flag,7]
  message,'Object entirely masked out',/info,noprint=silent
endif
n_neigh=n_elements(uniq(bits,sort(bits)))-2-keyword_set(circular)
if nothpix gt 0 then begin
    flag=[flag,2]       
   message,'Found '+strtrim(string(n_neigh),2)+' neighbour(s)',/info,noprint=silent
;  message,'Masking out a nearby object',/info,noprint=silent
;  bitn[othpix]=0.   ; Set weight of those pixels to zero
endif


; Contruct noise map in postage stamp region
; First, estimate background by considering pixels outside objects
bgpix = where(bits eq 0,nbgpix)
if nbgpix gt nbgpix_min then begin
  backg=mean(bit[bgpix])
  noise_rms=stddev(bit[bgpix])
  message,'Local background: mean, rms:'+string(backg)+string(noise_rms),/info,noprint=silent
endif else begin
  flag=[flag,6]
  backg=0.
  noise_rms=0.
  message,'Not enough pixels to evaluate local background',/info,noprint=silent
endelse
if keyword_set(noise_map) then begin         ; use supplied noise map if requested
  if tag_exist(image,'noise') then begin
    if n_elements(image.noise) eq 1 and image.noise[0] eq 0 then message,'Noise map not provided!'
    noise=image.noise[xmin:xmax,ymin:ymax]
    noise_mean=0.
    if nbgpix gt nbgpix_min then noise_mean=mean(1./sqrt(noise[bgpix]))
                           ; compare background estimation to provided map
    message,'Noise map: <rms>:'+string(noise_mean),/info,noprint=silent
  endif else begin
    message,'Noise map not provided!'
  endelse
endif else begin    ; use local background estimation
  noise_mean=noise_rms
  noise=replicate(1.,n_bit[0],n_bit[1])/noise_rms^2   ; constant inverse variance
endelse
if nothpix gt 0 then noise[othpix]=0.  ; blank out nearby objects
if keyword_set(circular) then $
  if n_corner gt 0 then noise[corner]=0.

; Normalise noise map locally
;bgpix = where(bits eq 0,nbgpix)
;if keyword_set(renorm) and nbgpix gt 10 then begin
;  nmnorm=(stddev(bit[bgpix]))^(-2) / mean(bitn[bgpix]) 
;  ;nmnorm=(stddev(bit[bgpix])*1.9)^(-2) / mean(bitn[bgpix]) ; 1.9 from Williams et al 1996
;  ;if (strmid(sexcat.name,0,3) eq "HDF") then nmnorm=nmnorm/0.28 ; from Williams et al 1996
;  if (strmid(sexcat.name,0,3) eq "HDF") then nmnorm=nmnorm/0.33 ; from Williams et al 1996
;  message,'Local normalisation of noise map: '+$
;    strtrim(nmnorm,1),/info,noprint=silent
;  bitn=bitn*nmnorm
;endif else flag=flag+1

; Convert centroid guess into postage-stamp coordinates
x0=[sexcat.x[id,0],sexcat.x[id,1]]-[xmin,ymin]

; Store in an "pstamp" structure
pstamp={name:image.name+"_#"+strtrim(id,2),$
        type:"pstamp",           $
        flag:max(flag),          $      ; 0=Fail, 1=OK, 2=Nearby obj, 4=Noise normalised
        image:bit,               $      ; Image data,
        noise:noise,             $      ; Pixel weight map
        seg:bits,                $      ; Image segmentation map
        im_ran:[xmin,xmax,ymin,ymax],$  ; Coordinates of BL and TR of postage stamp
        xo:x0[0],                $      ; Centroid guess in postage stamp coordinate
        yo:x0[1],                $      ;
        back_mean_local:backg,   $      ; local background estimation - mean
        back_rms_local:noise_rms,$      ;                             - rms
        back_rms:noise_mean,     $      ; rms background from external map (if available)
        sexid:id+1,              $      ; SExtractor ID number
        x:sexcat.x[id,0],        $      ; SExtractor centroid guess in image coordinates
        y:sexcat.x[id,1],        $      ;
        xpeak:sexcat.xpeak[id,0],$      ;
        ypeak:sexcat.xpeak[id,1],$      ;
        xmin:sexcat.xmin[id,0],  $      ;
        ymin:sexcat.xmin[id,1],  $      ;
        xmax:sexcat.xmax[id,0],  $      ;
        ymax:sexcat.xmax[id,1],  $      ; Other SExtractor
        area:sexcat.area[id],    $      ;  parameters, all in
        a:sexcat.a[id],          $      ;  image coordinates
        b:sexcat.b[id],          $      ;
        theta:sexcat.theta[id],  $      ;
        class:sexcat.class[id],  $      ;
        nu:sexcat.flux[id],      $      ;
        fwhm:sexcat.fwhm[id],    $      ;
        mag:sexcat.mag[id]}             ;


; Plot the postage stamp (and SExtractor parameters) to screen
if keyword_set(plotit) then plt_pstamp,pstamp,/mask,title='!6Postage stamp #'+strtrim[id,1];,/oframe

; Return postage stamp structure
return,pstamp

end
