function shapelets_shapecat2decomp, shapecat, id, $
                                    OVER=over, INTEGRATE=integrate, V1=v1

;$Id: shapelets_shapecat2decomp.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SHAPELETS_SHAPECAT2DECOMP
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Extracts one (decomp structure) object from a (shapecat structure)
;      catalogue.
;
; INPUTS:
;      SHAPECAT - Shapelet catalogue structure. Can contain thousands of
;                 objects.
;      ID       - If the catalogue contains more than one object, specify
;                 which one of them is to be converted.
;
; OPTIONAL INPUTS:
;      OVER     - Oversampling amount in decomp structure. Default: 1
;
; KEYWORD PARAMETERS:
;      INTEGRATE- Integrate basis functions within pixels. Default: 1
;      V1       - Assume shapelet coefficients are stored as a_ij/a_00, as
;                 in version 1 of shapelet code. Default is now just a_ij.
;
; OUTPUTS:
;      DECOMP   - Cartesian decomp structure containing all of the information
;                 that it is possible to reasonably extract from a catalogue.
;
; TO DO: 
;      Recover an object's shapelet moments from the shapelet catalogue.
;
; MODIFICATION HISTORY:
;      Jun 04 - Bug involving integers not longs fixed by RM.
;      Apr 04 - Convention on storage of aij/a00 to aij changed by RM.
;      Mar 04 - Tidied up by RM.
;      Apr 02 - Can now take polar catalogues/objects as input.
;      Dec 01 - Written by Richard Massey.
;-

; Initiliase variables
COMPILE_OPT idl2
on_error,2

n_pix_min=30
n_pix_max=100
if not keyword_set(id) then id=0
if id ge shapecat.n or id lt 0 then $
  message,"Object #"+strtrim(string(id),2)+" does not exist in this catalogue of only "+strtrim(string(shapecat.n),2)+" objects!"


; Extract shapelet meta-parameters from catalogue
beta=shapecat.beta[id]
n_max=shapecat.n_max[id]
shapelets_make_nvec,n_max,n1,n2,n_coeffs


; Extract coefficient list (and errors on coefficients) from catalogue
if shapecat.polar then begin
  ; Convert polar shapelet coefficients to Cartesian shapelet coefficients
  coeffs=(shapelets_p2c(shapelets_polar_expand(transpose(shapecat.coeffs[id,*]))))[0:n_coeffs-1] ; convert to cartesian
  coeffs_error=(shapelets_p2c(shapelets_polar_expand(transpose(shapecat.coeffs_error[id,*]))))[0:n_coeffs-1]
endif else begin
  coeffs=reform((transpose(shapecat.coeffs))[0:n_coeffs-1,id])
  coeffs_error=reform((transpose(shapecat.coeffs_error))[0:n_coeffs-1,id])
endelse


; Rescale coefficients from relative to a_00 overall brightness
; (STORAGE CONVENTION CHANGED APRIL 2004)
if keyword_set(v1) and n_coeffs gt 1 then coeffs[1:*]=coeffs[1:*]*coeffs[0]


; Guess (arbitrary) number of pixels for image display
th_max=beta*sqrt(n_max+0.5)
n_pixels=round([1.,1.]*1.5*th_max^1.1)+[10,10]
n_pixels[0]=max([n_pixels[0],n_pix_min]) & n_pixels[1]=max([n_pixels[1],n_pix_min])
n_pixels[0]=min([n_pixels[0],n_pix_max]) & n_pixels[1]=min([n_pixels[1],n_pix_max])
x=n_pixels/2.


; Recover sextractor information from shapelet catalogue if available
sextractor = 0B
if tag_exist(shapecat,"sextractor") then begin
  sextractor=shapecat.sextractor
  if sextractor eq 1B then begin
    sex={seeing:shapecat.seeing,                                $
         x:[shapecat.sexx[id],shapecat.sexy[id]],               $
         xmin:[shapecat.objx_min[id],shapecat.objy_min[id]],    $
         xmax:[shapecat.objx_max[id],shapecat.objy_max[id]],    $
         id:shapecat.sexid[id],                                 $
         class:shapecat.sexclass[id],                           $
         a:shapecat.sexa[id],                                   $
         b:shapecat.sexb[id],                                   $
         theta:shapecat.sextheta[id],                           $
         e:sqrt(shapecat.sexe1[id]^2+shapecat.sexe2[id]^2),     $
         e1:shapecat.sexe1[id],                                 $
         e2:shapecat.sexe2[id],                                 $
         flux:shapecat.sexflux[id],                             $
         mag:shapecat.sexmag[id],                               $
         fwhm:shapecat.sexfwhm[id],                             $
         area:shapecat.sexarea[id] }
  endif
endif


; Recover object's moments from shapelet catalogue, to store in decomp.moments
;
; TO DO!
;
moments=0B


; Last bit of meta-information about the object's decomposition
if n_elements(over) eq 0      then over      = 1
if n_elements(integrate) eq 0 then integrate = 1B


; store in a structure
decomp = {name:shapecat.name+"_"+strtrim(string(id),2), $
          type:"decomp_cartesian",                      $
          history:"Object #"+strtrim(string(id),2)+" from catalogue "+shapecat.name,$
          x:x,                     		           $
          beta:beta,               		           $
          n_max:fix(n_max),            		           $
          n_coeffs:n_coeffs,       		           $
          coeffs:coeffs,           		           $
          coeffs_error:coeffs_error,       	           $
          n_pixels:n_pixels,       		           $
          sextractor:sextractor,      		           $
          moments:moments,           		           $
          n1:n1,                   		           $
          n2:n2,                   		           $
          over:over,               		           $
          integrate:integrate,     		           $
          chisq:[0.,0.],           		           $
          skyfit:0}
if sextractor eq 1B then decomp=create_struct(decomp,"sex",sex)

return,decomp

end
