function shapelets_convolve,decomp,psf,gamma=gamma,n_max_g=n_max_g,silent=silent

;$Id: shapelets_convolve.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SHAPELETS_CONVOLVE
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Convolves one image with another (e.g. a PSF).
;
; INPUTS:
;      DECOMP - A shapelet decomp structure representing the first image.
;      PSF    - PSF model, as a Cartesian decomp structure
;
; OPTIONAL INPUTS:
;      GAMMA  - Gamma to use for convolved object.
;               default: "natural choice" as described in Shapelets II.
;      N_MAX  - N_max to use for convolved objecy.
;               default: same as input object.
;
; KEYWORD PARAMETERS:
;      None.
;
; OUTPUTS:
;      DECOMP - returns convolved object, as a Cartesian decomp structure
;
; MODIFICATION HISTORY:
;      Jul 04 - Bug fixed (new object didn't get new beta) by RM.
;      Jun 04 - Error on coefficients and history tags included by RM.
;      Mar 02 - Written by Richard Massey.
;-

; Set up various size parameters
alpha=decomp.beta & beta=psf.beta
if not keyword_set(gamma) then gamma=sqrt(alpha^2+beta^2)
n_max_a=decomp.n_max
if not keyword_set(n_max_g) then n_max_g=decomp.n_max

; Convolution (paperI eqn54)
P_nm = shapelets_convolution_matrix(psf,alpha,n_max_a,gamma,n_max_g)
f_m  = decomp.coeffs
h_n  = P_nm#f_m

; A long-winded check by hand
;h_n=fltarr(nn_g)
;h_error=fltarr(nn_g)
;for n=0,nn_g-1 do begin
;  for m=0,decomp.n_coeffs-1 do begin
;    h_n(n)=h_n(n)+P_nm(n,m)*f_m(m)
;    h_error(n)=h_error(n)+P_nm(n,m)*decomp.error(m)
;  endfor
;endfor

; Might as well do errors at the same time (do these transform in exactly the same way?)
h_n_error=P_nm#decomp.coeffs_error
;n_a=n_elements(decomp.coeffs)
;if n_elements(decomp.error) eq n_a $
;  then h_error=P_nm#decomp.error $
;  else h_error=fltarr(n_a)

; Write results into a new structure for output
output = decomp                              ; Simply copy the structure from the one used before
shapelets_extend_nmax,output,n_max_g-n_max_a,silent=silent ; Set its new n_max
output.coeffs=h_n                            ; Copy the new shapelet coefficients into the structure (keeping flux constant when we change beta in a moment))
output.coeffs_error=h_n_error                ; Copy the error on the new shapelet coefficients
output.beta=gamma                            ; Set it's new beta

; Add operation in object's history record
if tag_exist(output,"history") then begin
 output.history=[output.history+" Convolved with "+psf.name+"."]
endif else begin
 output=create_struct(output,"history","Convolved with "+psf.name)
endelse

return,output

end
