pro shapelets_extend_nmax,decomp,n_increase,SILENT=silent

;$Id: shapelets_extend_nmax.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SHAPELETS_EXTEND_NMAX
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Increases the value of n_max in a structure by n_increase, by padding
;      the coefficient arrays with zeros. Can also decrease n_max by truncating
;      the coefficient arrays.
; 
; INPUTS:
;      decomp     - Cartesian shapelet decomp structure.
;      n_increase - (Positive or negative) integer amount by which to change
;                   n_max.
;
; OPTIONAL INPUTS:
;      None.
;
; KEYWORD PARAMETERS:
;      silent - Operate silently.
;
; OUTPUTS:
;      decomp - Cartesian shapelet decomp structure with new n_max.
; 
; MODIFICATION HISTORY:
;      Jun 04 - Made robust to changes of tag names in decomp structures by RM.
;      Apr 02 - Written by Richard Massey
;-

COMPILE_OPT idl2

; parse input requirements
if not keyword_set(n_increase) then begin
  message,'not changing n_max!',/info,noprint=silent
  n_increase=0
endif

; determine increased number of coefficients
n_max = decomp.n_max+n_increase
shapelets_make_nvec,n_max,n1,n2,n_coeffs
; and use a rather ad hoc rule to increase the number of pixels in recomp
sizeinc = sqrt((n_max+1)/(decomp.n_max+1))  

; pad coefficient arrays with zeros and change n_max etc in decomp structure
decomp_new={name:decomp.name,type:decomp.type}
for i=0,n_tags(decomp)-1 do begin
  tagname=strupcase((tag_names(decomp))[i])
  case strlowcase((tag_names(decomp))[i]) of
    "name":
    "type":
    "coeffs":       begin
                      coeffs=decomp.coeffs
                      while n_elements(coeffs) lt n_coeffs do coeffs = [coeffs,0.]
                      if n_elements(coeffs) gt n_coeffs then coeffs = coeffs[0:n_coeffs-1]
                      decomp_new=create_struct(decomp_new,"coeffs",coeffs)
                    end
    "coeffs_error": begin
                      coeffs_error=decomp.coeffs_error
                      while n_elements(coeffs_error) lt n_coeffs do coeffs_error = [coeffs_error,0.]
                      if n_elements(coeffs_error) gt n_coeffs then coeffs_error = coeffs_error[0:n_coeffs-1]
                      decomp_new=create_struct(decomp_new,"coeffs_error",coeffs_error)
                    end
    "error":        begin
                      error=decomp.error
                      while n_elements(error) lt n_coeffs do error = [error,0.]
                       if n_elements(error) gt n_coeffs then error = error[0:n_coeffs-1]
                     decomp_new=create_struct(decomp_new,"error",error)
                    end
    "n_max": decomp_new=create_struct(decomp_new,"n_max",n_max)
    "n_coeffs": decomp_new=create_struct(decomp_new,"n_coeffs",n_coeffs)
    "n1": decomp_new=create_struct(decomp_new,"n1",n1)
    "n2": decomp_new=create_struct(decomp_new,"n2",n2)
    "x":decomp_new=create_struct(decomp_new,"x",decomp.x + round(sizeinc)*[1.,1.])
    "n_pixels":decomp_new=create_struct(decomp_new,"n_pixels",decomp.n_pixels + 2*round(sizeinc)*[1,1])
    else: decomp_new=create_struct(decomp_new,tagname,decomp.(i))
  endcase
endfor
decomp=decomp_new

end
