pro shapelets_translate,decomp,translation,extend=extend,polar=polar

;$Id: shapelets_translate.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SHAPELETS_TRANSLATE
;
; CATEGORY:
;      Shapelets.
;
; PURPOSE:
;      Translates an object (to first order, using ladder operators).
; 
; INPUTS:
;      decomp      - Cartesian shapelet decomp structure.
;      translation - [x,y] floating point amount by which to shift object.
;
; OPTIONAL INPUTS:
;      EXTEND - Integer value bu which to increase n_max by padding with zeros
;               before performing shft. If extend=1, default is n_max=n_max+4.
;
; KEYWORD PARAMETERS:
;      POLAR  - Perform translation in polar shapelet space. Default: Cartesian.
;
; OUTPUTS:
;      DECOMP - Shifted Cartesian shapelet decomp structure.
; 
; MODIFICATION HISTORY:
;      Apr 02 - written by Richard Massey
;-

; Extend n_max first if requested
if keyword_set(extend) then begin
  if extend eq 1 then shapelets_extend_nmax,decomp,4 else $
     shapelets_extend_nmax,decomp,round(extend)
endif

; Perform translation
if not keyword_set(polar) then begin

  ; Perform translation using Cartesian step operators
  n1=decomp.n1
  n2=decomp.n2
  new_coeffs=decomp.coeffs
  for i=0,decomp.n_coeffs-1 do begin

    ; delta_x
    ; Expand rightwards in Cartesian shapelet coefficient space
    j=where(n1 eq n1[i]+1 and n2 eq n2[i])
    if j[0] ne -1 then new_coeffs[i]=new_coeffs[i]-translation[0]*sqrt((n1[i]+1.)/2.)*decomp.coeffs[j[0]]

    ; Expand leftwards in Cartesian shapelet coefficient space
    j=where(n1 eq n1[i]-1 and n2 eq n2[i])
    if j[0] ne -1 then new_coeffs[i]=new_coeffs[i]+translation[0]*sqrt(float(n1[i])/2.)*decomp.coeffs[j[0]]

    ; delta_y
    ; Expand upwards in Cartesian shapelet coefficient space
    j=where(n1 eq n1[i] and n2 eq n2[i]+1)
    if j[0] ne -1 then new_coeffs[i]=new_coeffs[i]-translation[1]*sqrt((n2[i]+1.)/2.)*decomp.coeffs[j[0]]

    ; Expand downwards in Cartesian shapelet coefficient space
    j=where(n1 eq n1[i] and n2 eq n2[i]-1)
    if j[0] ne -1 then new_coeffs[i]=new_coeffs[i]+translation[1]*sqrt(float(n2[i])/2.)*decomp.coeffs[j[0]]

  endfor  

  decomp.coeffs=new_coeffs
 
endif else begin

  ; Convert to polar shapelet coefficients
  polar_coeffs=shapelets_c2p(decomp.coeffs) & new_polar_coeffs=polar_coeffs
  m=decomp.n1-decomp.n2
  n=decomp.n1+decomp.n2
  
  ; Perform translation in polar shapelet space
  for i=0,decomp.n_coeffs-1 do begin

    j=where(m eq m[i]-1 and n eq n[i]-1)
    if j[0] ne -1 then new_polar_coeffs[i] = new_polar_coeffs[i] + $
      ( complex(translation[0],translation[1]) * $
                                          sqrt(float(n[i]+m[i])/8.) * polar_coeffs[j[0]] ) 

    j=where(m eq m[i]+1 and n eq n[i]-1)
    if j[0] ne -1 then new_polar_coeffs[i] = new_polar_coeffs[i] + $
      ( complex(translation[0],-1*translation[1]) * $
                                          sqrt(float(n[i]-m[i])/8.) * polar_coeffs[j[0]] ) 

    j=where(m eq m[i]-1 and n eq n[i]+1)
    if j[0] ne -1 then new_polar_coeffs[i] = new_polar_coeffs[i] - $
      ( complex(translation[0],translation[1]) * $
                                          sqrt(float(n[i]-m[i]+2.)/8.) * polar_coeffs[j[0]] ) 

    j=where(m eq m[i]+1 and n eq n[i]+1)
    if j[0] ne -1 then new_polar_coeffs[i] = new_polar_coeffs[i] - $
      ( complex(translation[0],-1*translation[1]) * $
                                          sqrt(float(n[i]+m[i]+2.)/8.) * polar_coeffs[j[0]] ) 

  endfor

  ; Convert back to Cartesian shapelet coefficients
  decomp.coeffs=shapelets_p2c(new_polar_coeffs)

endelse

return

end

