pro plt_image,a,FRAME=frame,COLBAR=colbar,CRAN=cran,$
  TITLE=title,XTITLE=xtitle,YTITLE=ytitle,CTITLE=ctitle,$
  INVERSE=inverse,SCALABLE=scalable,NOERASE=noerase,$
  CSIZE=csize,ISOTROPIC=isotropic

;$Id: plt_image.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      PLT_IMAGE
;
; PURPOSE:
;      Draws a 2D pixellated image.
;
; CATEGORY:
;      Plotting.
;
; PURPOSE:
;      Plot an image scaled to the current coordinate frame.
;      Further plotting (such as contours) can be performed over the
;      resulting plot. Optionally, an annotated color bar can be drawn at
;      the top. The 'scalable' keyword must be invoked when outputing to a
;      postscript file.
;
; NOTES:
;      !p.region=0 can be used to reset the window after using this
;      routine
;
; INPUTS:
;      a - image array
;
; OPTIONAL INPUTS:
;      cran     - range of a for colors (default: [min(a),max(a)])
;      title    - title for the plot
;      x,ytitle - titles for the frame
;      ctitle   - title for the color bar
;	  csize    - vertical size of the color bar (0-1, default:.12)
;
; KEYWORD PARAMETERS:
;      noerase  - supress erasing whatever was previously in the window
;      colbar   - draw color bar
;      frame    - draw a frame with pixel index limits;
;                 to change limits set frame=[x0,x1,y0,y1].
;                 set frame=-1 to supress the frame while keeping
;                 the image within it.
;      inverse  - invert the color coding.
;      scalable - use scalable pixels (to be used with ps devices)
;                 (default: nonscalable to be used with x-term device).
;      isotropic- force scales on x and y axes to be identical.
;
; OUTPUTS:
;      Plot of the scaled image with, optionally, a coordinate frame
;      and a color bar.
;
;
; MODIFICATION HISTORY:
;      Jul 04 - Automatic detection of device type (scalable pixels?) by RM.
;      Nov 02 - Isotropic keyword added by R. Massey.
;      Aug 00 - Plotting over the full plotting region allowed by AR.
;    3 Sep 97 - Active changes for the color bar rangeadded by AR.
;   14 Oct 97 - Plotting of a color bar and flexible frame labels added by AR.
;    7 Jul 97 - Written by A. Refregier.
;-

; figure out what kind of plot we are making
; (if we're plotting to postscript, the pixels should be scalable)
if n_elements(scalable) eq 0 then $
  if strupcase(!d.name) eq "PS" then scalable=1 else scalable=0

; set range of a for colors
if not keyword_set(cran) then cran=[min(a),max(a)]

; draw color bar
if keyword_set(colbar) then begin
  plt_colbar,cran,scalable=scalable,inverse=inverse,title=ctitle,$
    csize=csize
  noerase=1
endif

; set up the coordinate frame
nx=n_elements(a(*,0))
ny=n_elements(a(0,*))
plot,[0],[0],/nodata,xstyle=12,ystyle=12,noerase=noerase

; map image to color indices
if not keyword_set(cran) then begin
  if keyword_set(inverse) then b=-a else b=a
  b_ran=[min(b),max(b)]
  b=(b-b_ran(0))/(b_ran(1)-b_ran(0))*(!d.table_size-1)    ; this is equivalent to tvscl
endif else begin
  if keyword_set(inverse) then begin    
    b=(1.-((a>cran(0)<cran(1))-cran(0))/(cran(1)-cran(0)))*(!d.table_size-1)
  endif else begin
    b=((a>cran(0)<cran(1))-cran(0))/(cran(1)-cran(0))*(!d.table_size-1)
  endelse
endelse

; plot image scaled to the frame size
if not keyword_set(scalable) then begin
; non-scalable pixel case (to be used for xterm device)
  if keyword_set(frame) then begin
    px=!x.window*!d.x_vsize     ; position of window in device pixels
    py=!y.window*!d.y_vsize
  endif else begin
    px=!x.region*!d.x_vsize     ; position of window in device pixels
    py=!y.region*!d.y_vsize    
  endelse
  sx=abs(px(1) - px(0) +1)         ; desired size of image in pixels
  sy=abs(py(1) - py(0) +1)
  if keyword_set(isotropic) then begin
    sx=min([sx,sy])
    sy=sx
  endif
  if keyword_set(isotropic) then begin
    print,'Warning: isotropic plotting not quite figured out yet'
    nmax=float(max([nx,ny])) ;& nx=float(nx) & ny=float(ny)
   !p.region=[ $
       !p.region(0), $
	  !p.region(1), $
       !p.region(0)+(!p.region(2)-!p.region(0))*nx/nmax, $
	  !p.region(1)+(!p.region(3)-!p.region(1))*ny/nmax ]
    sx=sx*(nx/nmax)
    sy=sy*(ny/nmax)
  endif
  tv,congrid(b,sx,sy),px(0),py(0)
endif else begin
; scalable pixel case (to be used for ps device)
  if keyword_set(frame) then begin  
    tv,b,$
      !x.window(0),!y.window(0),$
      xsize=!x.window(1)-!x.window(0),$
      ysize=!y.window(1)-!y.window(0),/norm
  endif else begin
    tv,b,$
      !x.region(0),!y.region(0),$
      xsize=!x.region(1)-!x.region(0),$
      ysize=!y.region(1)-!y.region(0),/norm 
  endelse   
endelse	

; draw frame if requested
if keyword_set(frame) then begin
  fsupress=0
  if n_elements(frame) eq 4 then begin
    xr=[frame(0),frame(1)] & yr=[frame(2),frame(3)]
  endif else begin
    if frame eq -1 then begin
      fsupress=1 
    endif else begin
      xr=[0.,nx] & yr=[0.,ny]
    endelse
  endelse
  if fsupress eq 0 then $
    plot,[0],[0],/nodata,/noerase,/xstyle,/ystyle,$
      xrange=xr,yrange=yr,$
      title=title,xtitle=xtitle,ytitle=ytitle
endif

;!p.region=[0.,0.,1.,1.]

end


