pro plt_polar, decomp, nran=nran, coef=coef, top=top, ps=ps,$
  cran=cran, title=title, frame=frame, pow=pow, scalable=scalable,$
  format=format, rotate=rotate

;$Id: plt_polar.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; Written by Richard Massey - Feb 02
;
; PURPOSE: takes a regular Cartesian decomp structure, converts it into
;          polar shapelets and plots the {n,m} coefficient array
;
; INPUT:
;         decomp: (Cartesian) structure produced by shapelets_decomp
;         [nran]: ignore coefficients with n outsdide of nran
;   dnf    [top]: keep top largest coefficients
;   dnf    [pow]: overlay shapelet coefficient power spectrum
;         format: 1 - magnitude, 2 - phase, 3 - mixture (top half mags, bottom half phases)
;
; OUTPUT:
;         a plot
;-

COMPILE_OPT idl2, HIDDEN

; Set defaults
if not keyword_set(format) then format=1


; Convert to polar shapelet coefficients
polar_coeffs=shapelets_c2p(decomp.coeffs,matrix=matrix)


; Rotate so major axis is horizontal
if keyword_set(rotate) then begin
  ; Find angle to major axis of nearest ellipse, using polar coefficients
  ;   This should be...
  ;e1=sqrt(2.)*float(a[5])/float(a[4])
  ;e2=2.*imaginary(a[5])/float(a[4])      ; ...but...
  ;   V--- extra sqrt(2) from different normalisation of quadrupole moments ???
  e1=double(polar_coeffs[5])/double(polar_coeffs[4])    ; Not normalised properly, so |e|
  e2=imaginary(polar_coeffs[5])/double(polar_coeffs[4]) ;   is wrong, but angle is OK.
  angle=atan(e2,e1)/2.
  ;print,'Angle to major axis is',angle*180./!pi,' degrees'
  decomp_rotate=decomp   ; Save input decomp structure, so can restore it later
  shapelets_rotate,decomp_rotate,-1.*angle,matrix=matrix
  polar_coeffs=matrix#decomp_rotate.coeffs
endif


; Top and tail coefficients to display only a certain n-range, if required
if keyword_set(nran) then begin
  ; truncate top end
  n_max=min([nran[1],decomp.n_max])
  n_coeffs=(n_max+1)*(n_max+2)/2.
  polar_coeffs=polar_coeffs[0:n_coeffs-1]
  ; don`t remove, but set low end to zero, to preserve triangle structure
  if nran[0] gt 0 then a[0:(nran[0])*(nran[0]+1)/2.-1]=complex(0.,0.)
endif else begin
  n_coeffs=decomp.n_coeffs
  n_max=decomp.n_max
endelse


; This is in {n_l,n_r} format - now convert this to {n,m}
coefs=dcomplexarr(n_max+1,2*n_max+2)
shapelets_make_nvec,n_max,n_r,n_l,n_coeffs
n=n_r+n_l
m=n_r-n_l
marray=fltarr(n_max+1,2*n_max+2)
for i=0,n_coeffs-1 do begin
  coefs[n[i],m[i]+n_max]=polar_coeffs[i];*exp(complex(0,1)*m[i]/2.*angle)
  coefs[n[i],m[i]+n_max+1]=polar_coeffs[i];*exp(complex(0,1)*m[i]/2.*angle)
  marray[n[i],m[i]+n_max]=m[i]
  marray[n[i],m[i]+n_max+1]=m[i]
endfor



; Now convert this into polar r and (relative) theta form
magnitude=abs(coefs)
argument=atan(imaginary(coefs),float(coefs))


; Take into account rotational symmetries (spin) of basis functions
;for i=0,n_max do for j=n_max-i,2*n_max+1 do $
;  if marray(i,j) ne 0 then argument(i,j)=argument(i,j) mod (!pi/(marray(i,j)))
; Make the phases RELATIVE (to what?!)
;for i=n_max,0,-1 do for j=n_max-i,n_max-1+i do $
;  argument(i,j)=(argument(i,j)-argument(2,n_max-2)) mod (!pi/2.)
;for j=n_max+1,n_max+1-i,-1 do begin
;  for i=1,n_max do begin
;    if abs(marray(i,j)) ge 1. then begin
;      aa(i,j)=(argument(i,j)-argument(i-1,j+1)) mod !pi
;    endif else if abs(marray(i,j)) eq 1. then begin
;      argument(i,j)=(argument(i,j)-argument(i-2,j)) mod !pi
;    endif
;
;  endfor
;endfor



; Options of what to plot
; could also imagine things like aa=imaginary(coefs)
if format eq 1 then begin
  aa=magnitude
  if not keyword_set(cran) then cran=[min(aa),max(aa)]
endif else if format eq 2 then begin
  aa=argument
  cran=[-1.*!pi,!pi]
endif else begin
  aa=fltarr(n_max+1,2*n_max+2)
  cran=[-1.*!pi,!pi]

  aa[*,0:n_max]=argument[*,0:n_max] ; Bottom half
  ;aa=(abs(aa)-!pi)*sign(aa)        ; reorganise colour-map for phases

  magnitude=magnitude*!pi/max(abs(magnitude))

  ;aa=aa/!pi*max(magnitude)*2      ; rescale colour-map for phases

  aa[*,n_max+1:2*n_max+1]=magnitude[*,n_max+1:2*n_max+1]  ; Top half
  for i=0,n_max,2 do aa[i,n_max]=magnitude[i,n_max]       ; Top half
endelse

; Open ps file if requested
if keyword_set(ps) then begin
  ops,form=1
  csize=.23
endif


; Plot coefficient array
plt_image,aa,/col,scal=ps,cs=csize,inv=ps,cran=cran,      $
  frame=[-.5,n_max+.5,-n_max-1,n_max+1],                  $
  xtitle='!6n',ytitle='!6m',title=title


; Close ps file if appropriate
if keyword_set(ps) then cps


end









