pro plt_stars,cat,simple=simple,class_c=class_c,mi_ran=mi_ran

; August 998 - Written by A. Refregier
;
; PURPOSE: plot various figures relative to the stars in a catalog.
; This is a measure of the PSF.
; INPUT: cat: catalog read by rd_cat.pro
; OPTIONAL INPUT: simple: simplify by only plotting stellar ellipticity
;      class_c: cut for star/galaxy classifier (default=0.8)
;      mi_ran: magnitude range for selected stars (default=[-15,-11])
; OUTPUT: plot
  
; declarations
if not keyword_set(class_c) then class_c = 0.8  ; class cut for stars
if not keyword_set(mi_ran) then mi_ran = [min(cat.mag), max(cat.mag)] 
                              ; magnitude range for useful stars
;th_pix=cat.th_pix  
npix_smooth=400               ; smoothing scale for PSF smoothing [pixels]

; select useful (unsaturated bright) stars
st=where(cat.class gt class_c and cat.mag gt mi_ran(0) and $
   cat.mag lt mi_ran(1),ns)
print,'No. of stars selected: ',ns


; plot mag-size relation
if not keyword_set(simple) then begin
plot, cat.mag, cat.fwhm, psym=3, yrange=[.1,1000],xrange=mi_ran,$
   xtitle='mag', ytitle='FWHM (pixels)',/ytype
oplot,cat.mag(st), cat.fwhm(st),psym=5
xyouts, /normal, .2, .9, 'No. of stars:'+string(ns)
xyouts, /normal, .2, .86, 'Class cut:'+string(class_c)
xyouts, /normal, .2, .82, 'mag range:'+string(mi_ran(0), mi_ran(1))

; plot size distribution
read, test
plothist,cat.fwhm(st),bin=.1,$
  xtitle='FWHM (pixels)',ytitle='No. of stars'
fwhm_mean = amean(cat.fwhm(st))
xyouts, /normal, .2, .9, 'mean FWHM (pixels)'+$
    string(fwhm_mean)
print,'mean FWHM (pixels):', fwhm_mean

; plot ellipticities
read,test
endif
plot,cat.e1(st),cat.e2(st),xrange=[-1., 1.], yrange=[-1., 1.], psym=3,$
  xtitle='e1',ytitle='e2'
oplot, [-5., 5.], [0., 0.], lines=1
oplot, [0., 0.], [-5., 5.], lines=1
e1_m = amean(cat.e1(st)) &  e2_m=amean(cat.e2(st))
e1_sig = asigma(cat.e1(st)) & e2_sig = asigma(cat.e2(st))
e1_err = e1_sig/sqrt(ns) &  e2_err= e2_sig/sqrt(ns) 
xyouts, /normal, .2, .9, 'e1,e2: mean:'+string(e1_m, e2_m)
xyouts, /normal, .2, .86, 'e1,e2: error:'+ string(e1_err, e2_err)
xyouts, /normal, .2, .82, 'e1,e2: rms:'+ string(e1_sig, e2_sig)
ploterr_xy, e1_m, e2_m, e1_err, e2_err
print,'e1,e2:'
print,'  mean:',e1_m, e2_m
print,'  error:',e1_err, e2_err
print,'  sigma:',e1_sig, e2_sig

; plot ellipticities as a function of position on the chip
read,test
xy_ran = [min([cat.x, cat.y]), max([cat.x, cat.y])]
plot,cat.x(st),cat.y(st),psym=3,xran=xy_ran, yran=xy_ran, /xstyle, /ystyle, $
  xtitle='x (pixels)',ytitle='y (pixels)'
scale=300.
plt_evec,cat.x(st),cat.y(st),sqrt(cat.e1(st)^2+cat.e2(st)^2),$
  cat.alpha(st)-!pi/2.,xs=scale,ys=scale

; compute and plot smoothed PSF ellipticity map
n_xs=fix((cat.x_ran(1)-cat.x_ran(0))/npix_smooth)
n_ys=fix((cat.y_ran(1)-cat.y_ran(0))/npix_smooth)
xs=(indgen(n_xs)+.5)*npix_smooth+cat.x_ran(0)
ys=(indgen(n_ys)+.5)*npix_smooth+cat.y_ran(0)
e1_s=fltarr(n_xs,n_ys) & e2_s=fltarr(n_xs,n_ys)
n_gd=fltarr(n_xs,n_ys)
for i=0l,ns-1 do begin
  is=fix((cat.x(st(i))-cat.x_ran(0))/npix_smooth)
  js=fix((cat.y(st(i))-cat.y_ran(0))/npix_smooth)
  if is ge 0 and is lt n_xs and js ge 0 and js lt n_ys then begin
    n_gd(is,js)=n_gd(is,js)+1l
    e1_s(is,js)=e1_s(is,js)+cat.e1(st(i))
    e2_s(is,js)=e2_s(is,js)+cat.e2(st(i))
  endif
endfor
e1_s=e1_s/float(n_gd)
e2_s=e2_s/float(n_gd)
read,test
plot,[0],[0],psym=3,xran=xy_ran, yran=xy_ran, /xstyle, /ystyle, $
  xtitle='x (pixels)',ytitle='y (pixels)',/nodata
scale=300.
e_scale=npix_smooth/.5
for i=0,n_xs-1 do begin
  for j=0,n_ys-1 do begin
    if n_gd(i,j) gt 0 then begin
      alpha=atan(e2_s(i,j),e1_s(i,j))/2.
      e=sqrt(e1_s(i,j)^2+e2_s(i,j)^2)
      oplot,xs(i)+e*cos(alpha)*e_scale*[-1.,1.],$
           ys(j)+e*sin(alpha)*e_scale*[-1.,1.]
    endif
  endfor
endfor
gds=where(n_gd gt 0,n_gds)
e1s_m = amean(e1_s(gds)) &  e2s_m=amean(e2_s(gds))
e1s_sig = asigma(e1_s(gds)) & e2s_sig = asigma(e2_s(gds))
e1s_err = e1s_sig/sqrt(n_gds) &  e2s_err= e2_sig/sqrt(n_gds) 
print,'smoothed e1,e2:'
print,'  mean:',e1s_m, e2s_m
print,'  error:',e1s_err, e2s_err
print,'  sigma:',e1s_sig, e2s_sig

; plot PSF ellipse as function of position on the chip
if not keyword_set(simple) then begin
read, test
plot,cat.x(st),cat.y(st),psym=3,xran=xy_ran, yran=xy_ran, /xstyle, /ystyle, $
  xtitle='x (pixels)',ytitle='y (pixels)', /nodata
plt_ellipse,cat.x(st),cat.y(st), cat.a(st), cat.b(st), cat.alpha(st)-!pi/2., $
  xs=30., ys=30.



; plot psf size vs x and y
read, test
!p.multi = [0, 1, 2]
plot, cat.x(st), cat.d(st), psym=3, yrange=[0.,4.],$
  xtitle='x (pixels)',ytitle='d (pixels)'
;slice_fit, cat.x(st), cat.d(st),  10,  5, x_sl, d_sl
;oplot, x_sl, d_sl
plot, cat.y(st), cat.d(st), psym=3,  yrange=[0.,4.],$
  xtitle='y (pixels)',ytitle='d (pixels)'
;slice_fit, cat.y(st), cat.d(st),  10,  5, y_sl, d_sl
;oplot, y_sl, d_sl
!p.multi = 0

endif

end
