function shapelets_asymmetry,decomp,        $
                             FLUX=flux,     $
                             MATRIX=matrix, $
                             ERROR=error
 
;$Id: shapelets_asymmetry.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SHAPELETS_ASYMMETRY
;
; PURPOSE:
;      Returns asymmetry morphology index of a Cartesian decomp structure.
;
; CATEGORY:
;      Shapelets.
;
; INPUTS:
;      DECOMP - A Cartesian shapelet decomposition.
;
; KEYWORD PARAMETERS:
;      ERROR  - Simultaneously calculate the error on the asymmetry.
;
; OUTPUTS:
;      ASYMMETRY - A floating point variable containing the object's asymmetry
;                  morphology index.
;                  If /ERROR is set, this becomes a vector
;                  [asymmetry,asymmetry_error]
;
; OPTIONAL OUTPUTS:
;      FLUX   - Flux of object (needed to calculate asymmetry anyway).
;      MATRIX - Cartesian to polar shapelet conversion matrix.
;
; EXAMPLE USE:
;      result = shapelets_asymmetry(decomp)
;
; MODIFICATION HISTORY:
;      Jun 04 - Written by Richard Massey.
;-

COMPILE_OPT idl2

; Calculate polar shapelet coefficients
polar_coeffs=float(shapelets_c2p(decomp.coeffs,matrix=matrix))
shapelets_make_nvec,decomp.n_max,n_r,n_l
m=n_r-n_l

; Calculate flux
flux=sqrt(!pi)*decomp.beta*2*total(polar_coeffs[where(m eq 0)])

; Calculate asymmetry
if flux eq 0 then asymmetry=0. else $
asymmetry=sqrt(2.)/!pi/flux*total(polar_coeffs[where(abs(m mod 2) eq 1)])

; Calculate error on asymmetry index if required
if keyword_set(error) then begin
  ; Errors will not be quite correct if we do not have the full covariance
  ;   matrix and try to perform the linear tranformation into polar space
  if size(decomp.coeffs_error,/n_dimensions) ne 1 then message,$
    "Warning: error calculation using covariance matrix is not yet written!"
  error_p=float(matrix#decomp.coeffs_error)
  flux_error=sqrt(!pi)*decomp.beta*2*sqrt(total((error_p^2)[i]))
  asymmetry_error=sqrt(2.)/!pi/flux_error*total(error_p[where(abs(m mod 2) eq 1)])
  flux=[flux,flux_error]
  asymmetry=[asymmetry,asymmetry_error]
endif

return,asymmetry

end


