function shapelets_centroid, decomp,        $
                             FLUX=flux,     $
                             ERROR=error,   $
                             MATRIX=matrix, $
                             POLAR=polar,   $
                             CARTESIAN=cartesian

;$Id: shapelets_centroid.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SHAPELETS_CENTROID
;
; PURPOSE:
;      Computes the centroid from a linear summation of shapelet coefficients,
;      read in from a Cartesian decomp structure.
;
; CATEGORY:
;      Shapelets.
;
; INPUTS:
;      decomp - A Cartesian shapelet decomp structure, with at least
;               beta and coefficients defined.
;
; KEYWORD PARAMETERS:
;     [/CART] - Perform summation in Cartesian shapelet space. [DEFAULT]
;    [/POLAR] - Perform sunnation in polar shapelet space.
;    [/ERROR] - Simultaneously calculate the error on the centroid (and flux).
;
; OUTPUTS:
;    centroid - Shapelet measure of centroid
;      [flux] - Floating point variable optionally returned with the object's 
;               flux (calculation needed to work out the centroid anyway).
;    [matrix] - Cartesian to polar conversion matrix, if /POLAR was set.
;
; EXAMPLE USE:
;      result = shapelets_centroid(decomp,flux=flux)
;
; NOTES: 
;      Not yet compatible with oversampling, nor full covariance error matrix
;      of coefficients.
;
; MODIFICATION HISTORY:
;      Oct 03 - Calculation of error on flux added by RM.
;      Feb 03 - Written by R.Massey
;-

COMPILE_OPT idl2

;
; SET DEFAULTS
;
if keyword_set(polar) then cartesian=0B else cartesian=1B


if cartesian then begin 

  ;
  ; WORK IN CARTESIAN SHAPELET SPACE
  ;
  
  fweight=fltarr(decomp.n_coeffs)
  xweight=fltarr(decomp.n_coeffs)
  yweight=fltarr(decomp.n_coeffs)

  for i=0,decomp.n_coeffs-1 do begin
    n1=decomp.n1[i] & n2=decomp.n2[i]
    if n1 mod 2 eq 0 and n2 mod 2 eq 0 then begin

      fweight[i]=$
       2.^(.5*(2.-n1-n2)) * $
       sqrt(factorial(n1)*factorial(n2))/factorial(n1/2)/factorial(n2/2)

    endif else if n1 mod 2 eq 1 and n2 mod 2 eq 0 then begin

      xweight[i]=sqrt(n1+1.)*2.^(.5*(2.-n1-n2))*$
       sqrt(factorial(n1+1)*factorial(n2))/$
       factorial((n1+1)/2)/factorial(n2/2)

    endif else if n1 mod 2 eq 0 and n2 mod 2 eq 1 then begin
      yweight[i]=sqrt(n2+1.)*2.^(.5*(2.-n1-n2))*$
       sqrt(factorial(n1)*factorial(n2+1))/$
       factorial(n1/2)/factorial((n2+1)/2)

    endif

  endfor

  rootpi=sqrt(!pi)

  flux=rootpi*decomp.beta*total(fweight*decomp.coeffs)
  xc=rootpi*decomp.beta^2*total(xweight*decomp.coeffs)/flux
  yc=rootpi*decomp.beta^2*total(yweight*decomp.coeffs)/flux
  centroid=[xc,yc]

  if keyword_set(error) then begin
    flux_error = rootpi*decomp.beta*sqrt(total((fweight*decomp.coeffs_error)^2))
    xc_variance= !pi*decomp.beta^4*total((xweight*decomp.coeffs_error)^2)
    xc_error   = xc*sqrt(xc_variance/(xc*flux)^2+(flux_error/flux)^2)
    yc_variance= !pi*decomp.beta^4*total((yweight*decomp.coeffs_error)^2)
    yc_error   = yc*sqrt(yc_variance/(yc*flux)^2+(flux_error/flux)^2)
    flux       = [flux,flux_error]
    centroid   = [[centroid],[xc_error,yc_error]]
  endif

endif else begin

  ;
  ; WORK IN POLAR SHAPELET SPACE
  ;
  
  polar_coeffs=shapelets_c2p(decomp.coeffs,matrix=matrix)
  polar_coeffsr=float(polar_coeffs)

  m=decomp.n1-decomp.n2
  n=decomp.n1+decomp.n2
  rootpi=sqrt(!pi)
  i0=where(m eq 0)
  i1=where(m eq 1)

  flux=rootpi*decomp.beta*2*total(polar_coeffsr[i0])
  centre=rootpi*decomp.beta^2*sqrt(8)*total((polar_coeffs*sqrt((n+1)))[i1])/flux
  xc=float(centre)  ;           ^--- 2 from weight calculation, 2 from m=1 and m=-1, 1/root(2) from inside summation
  yc=imaginary(centre)
  centroid=[xc,yc]

  if keyword_set(error) then begin
    if size(decomp.coeffs_error,/n_dimensions) ne 1 then message,$
      "Warning: error calculation using covariance matrix is not yet written!"
    error_p    = matrix#decomp.coeffs_error
    error_pr   = float(error_p)
    error_pi   = imaginary(error_p)
    flux_error = rootpi*decomp.beta*2*sqrt(total((error_pr^2)[i0]))
    xc_variance= !pi*decomp.beta^4*8*total(((error_pr*sqrt(n+1))[i1])^2)
    xc_error   = xc*sqrt(xc_variance/((xc*flux)^2)+(flux_error/flux)^2)
    yc_variance= !pi*decomp.beta^4*8*total(((error_pi*sqrt(n+1))[i1])^2)
    yc_error   = yc*sqrt(yc_variance/((yc*flux)^2)+(flux_error/flux)^2)
    flux       = [flux,flux_error]
    centroid   = [[centroid],[xc_error,yc_error]]
  endif

endelse

; Return a position relative to the bottom left of the postage stamp,
;   rather than the centre of the shapelet basis functions
if n_elements(decomp.x) gt 0 then centroid[0:1]=centroid[0:1]+decomp.x
return,centroid

end
