function shapelets_flux,decomp,        $
                        MATRIX=matrix, $
                        ERROR=error,   $
                        POLAR=polar,   $
                        CARTESIAN=cartesian
 
;$Id: shapelets_flux.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SHAPELETS_FLUX
;
; PURPOSE:
;      Returns total flux of a Cartesian decomp structure.
;      This does the same as shapelets_moments.pro but in function form.
;
; CATEGORY:
;      Shapelets.
;
; INPUTS:
;      decomp: A Cartesian shapelet decomposition.
;
; KEYWORD PARAMETERS:
;     [/CART] - Perform summation in Cartesian shapelet space. [DEFAULT]
;    [/POLAR] - Perform sunnation in polar shapelet space.
;    [/ERROR] - Simultaneously calculate the error on the flux.
;
; OUTPUTS:
;        flux - A floating point variable containing the object's total flux.
;               If /ERROR is set, this becomes a vector [flux,flux_error]
;    [matrix] - Cartesian to polar conversion matrix, if /POLAR was set.
;
; EXAMPLE USE:
;      result = shapelets_flux(decomp)
;
; NOTES: 
;      Not yet compatible with a full covariance error matrix of coefficients.
;
; MODIFICATION HISTORY:
;      Sep 03 - Calculation of error on flux added by RM.
;      Feb 02 - Written by Richard Massey.
;-

COMPILE_OPT idl2

;
; SET DEFAULTS
;
if keyword_set(polar) then cartesian=0B else cartesian=1B

if cartesian then begin 
  ;
  ; WORK IN CARTESIAN SHAPELET SPACE  
  ;

  weight=fltarr(decomp.n_coeffs)
  for i=0,decomp.n_coeffs-1 do begin
    n1=decomp.n1[i] & n2=decomp.n2[i]
    if n1 mod 2 eq 0 and n2 mod 2 eq 0 then begin
      weight[i]=2.^(.5*(2.-n1-n2))                 $
                *sqrt(factorial(n1)*factorial(n2)) $
                /factorial(n1/2)/factorial(n2/2)
    endif
  endfor
  flux=sqrt(!pi)*decomp.beta*total(weight*decomp.coeffs)

  if keyword_set(error) then begin
    flux_error=sqrt(!pi)*decomp.beta*sqrt(total((weight^2)*(decomp.coeffs_error^2)))
    flux=[flux,flux_error]
  endif

endif else begin
  
  ;
  ; WORK IN POLAR SHAPELET SPACE
  ;
  
  polar_coeffs=float(shapelets_c2p(decomp.coeffs,matrix=matrix))
  shapelets_make_nvec,decomp.n_max,n_r,n_l
  m=n_r-n_l
  i=where(m eq 0)
  flux=sqrt(!pi)*decomp.beta*2*total(polar_coeffs[i])

  if keyword_set(error) then begin
    ; Errors will not be quite correct if we do not have the full covariance
    ;   matrix and try to perform the linear tranformation into polar space
    if size(decomp.coeffs_error,/n_dimensions) ne 1 then message,$
      "Warning: error calculation using covariance matrix is not yet written!"
    error_p=float(matrix#decomp.coeffs_error)
    flux_error=sqrt(!pi)*decomp.beta*2*sqrt(total((error_p^2)[i]))
    flux=[flux,flux_error]
  endif

endelse

return,flux

end


