pro sex, filename, TELESCOPE=telescope, $
                   FILTER=filter,       $
                   FULLPATH=fullpath,   $
                   SILENT=silent,       $
                   NOBACKSEG=nobackseg

;$Id: sex.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.astro.caltech.edu/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;       SEX
;
; PURPOSE:
;       Runs SExtractor (by spawning it as a child process under the shell
;       used to run IDL) to locate objects within an image.
;
; CATEGORY:
;       ROI finder.
;
; CALLING PROCEDURE:
;       sex,"example"
;
; INPUTS:
;       filename - Name of input image
;
; OPTIONAL INPUTS:
;       telescope - Gets SExtractor parameters right. Default is HDF.
;       filter    - SExtractor convolution filter. Default is "gauss_2.0_5x5"
;
; KEYWORD PARAMETERS:
;       /SILENT   - Operates silently.
;       /NOBACKSEG- Will not output background maps and segmentation maps.
;                   These are otherwise written to disc by default.
;
; OUTPUTS:
;       SExtractor is run and outputs various files to disc.
;
; PROCEDURES USED:
;       None. But requires SExtractor to be installed on the computer, with
;       the full path of the "sex" executable stored in shapelets_paths.pro.
;
; MODIFICATION HISTORY:
;       Jul 04 - The segmentation file is now called _seg.fits by Alexandre Refregier
;       May 04 - Convention that weight maps stored as _weight.fits adopted by RM
;       Sep 03 - sex.pro adapted from shapelets_read_sexcat.pro by RM
;       May 03 - Ellipticity e,e1,e2 notation added to structure by RM
;       Mar 03 - Filepaths generalised and routine renamed shapelets_read_sexcat by RM
;       Apr 02 - rd_scat.pro adapted from shapelets_extractor.pro by RM
;       Dec 01 - First version of shapelets_extractor.pro written by Richard Massey
;-

COMPILE_OPT idl2
ON_ERROR,2


; Set defaults
if not keyword_set(silent)    then silent=0              ; Operate silently?
if not keyword_set(telescope) then telescope='HDF'	  ; Gets SEx params right
if not keyword_set(filter)    then filter='gauss_2.0_5x5'; SExtractor detection filter
if not keyword_set(filename)  then message,'You must specify a file name!'
file=filename                                            ; Don't disrupt input variables
if keyword_set(fullpath) then begin                      ;
  ; Strip file extension
  file=strmid(file,0,strpos(strlowcase(file),".fit"))
endif else begin
  ; Add relevant path name
  file=shapelets_paths(3,silent=silent)+file
endelse


; Check to see if the image file exists
if ( not file_test(file+'.fits') ) then $
	message,'Image '+file+'.fits not available!'


; Print intent to screen
message,'Locating objects with SExtractor',/info,noprint=silent


; Form shell command
sex_dir=shapelets_paths(7,/silent)
command_line=shapelets_paths(8,/silent)+' '+file+'.fits'+$
  ' -c '              +sex_dir+telescope+'.sex'+$
  ' -STARNNW_NAME '   +sex_dir+'default.nnw'   +$
  ' -FILTER_NAME '    +sex_dir+filter+'.conv'  +$
  ' -PARAMETERS_NAME '+sex_dir+'shex.param'    +$
  ' -CATALOG_NAME '   +file+'.sexcat '
if not keyword_set(nobackseg) then command_line=command_line +$
  '-CHECKIMAGE_TYPE "SEGMENTATION","-OBJECTS" '              +$
  '-CHECKIMAGE_NAME "'+file+'_seg.fits",'                +$
                   '"'+file+'_back.fits" '
if keyword_set(silent) then $
  command_line=command_line+'-VERBOSE_TYPE QUIET '
if ( file_test(file+'_weight.fits') ) then $
  command_line=command_line              +$
  '-WEIGHT_TYPE MAP_WEIGHT '             +$
  '-WEIGHT_IMAGE '+file+'_weight.fits'
;if (not file_test(file+'_weight.fits') ) then  $
;  command_line=command_line+'-CHECKIMAGE_TYPE BACKGROUND RMS'


; Run SExtractor
spawn,command_line


end
