;$Id: shex.pro, v2$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
; ***********************************************************************
; ***********************************************************************
;
;+
; NAME:
;        SHEX
;
; PURPOSE:
;        Having located objects within an image using SExtractor, this routine
;        now decomposes all of the objects into shapelets. It uses the
;        shapelets focus suite of routines to optimise the nmax, beta and
;        centroid parameters. All of the decompositions are written to disc
;        in a shapelets catalgoue. This can be later read to memory using
;        shapelets_read_sexcat.pro
;
; CATEGORY:
;        Shapelets/ROI modelling.
;
; CALLING PROCEDURE:
;        shex,'example',sexcat=sexcat,n_max=15,/plot
;
; INPUTS:
;        filename_input - file name of input image.
;
; OPTIONAL INPUTS:
;         IMAGE    - Can set in advance so it doesn't have to be loaded in.
;         SEXCAT   - Can set in advance so it doesn't have to be loaded in.
;         PSF      - Shapelet decompstructure containing PSF to deconvolve from.
;                    Currently, this has to be the same everywhere on an image.
;         SKY      - Allow sky background subtraction by fitting during image
;                    decomposition (1: locally-determined constant, 
;                    2: locally-determined plane, 3: global constant, 4: 3+1 &c.)
;	     N_MAX    - Highest order of n_max=n1+n2 used for decomposition.
;	     N_MIN    - Minimum value of n_max=n1+n2 used for decomposition.
;         SG_CUT   - Threshold level for star/galaxy separation, using the 
;                    SExtractor stellar clasification from {0,1}. If this is
;                    positive, only galaxies with SExtractor values lower
;                    than sg_cut are decomposed. If this is negative, only
;                    stars with SExtractor values greater than -sg_cut are 
;                    decomposed. If not set or zero, everything is decomposed.
;	     TOO_BIG  - Maximum radius of postage stamp in pixels (objects that
;                    look as if they are bigger than this get skipped). Can set
;                    so that IDL doesn't run out of memory.
;         INDEX    - Vector of SExtractor index number to process 
;                    (Default: all of the SExtractor catalogue is used). Note
;                    that the SExtractor index is equal to sextractor ID-1
;         IM_IN    - Additional input image for plotting purposes only (e.g. 
;                    noise-free simulated image, or image in a second colour)
;         SCAT_IN  - Input scat catalogue for plotting purposes only
;                    (e.g. noise free input catalogue from simulations)
;
; KEYWORD PARAMETERS:
;	     SILENT   - Operates silently.
;	     VERBOSE  - Operates noisily (VERBOSE=2 to operate very noisily).
;	     PLOTIT   - Display pretty pictures to screen.
;         SEG_MAP  - Use supplied segmentation map stored in postage stamp.
;         NOISE_MAP- Use supplied noise map stored in postage stamp.
;         CIRCULAR - Use a circular postage stamp (default is square).
;
; OUTPUTS:
;        Shapelet catalogue written to disc, with the same file name and path
;        as the input image, but with the extension .shapecat.
;
; OPTIONAL OUTPUTS:
;        SHAPECAT  - Shapelet catalogue data structure stored to memory.
;                    View contents by typing "help,cat,/structure"
;	    IMAGE     - Image data structure, if none was entered originally.
;	    SEXCAT    - SExtractor catalogue data structure, if none was entered.
;
; DATA REQUIRED:
;        Needs an image and a SExtractor catalogue to initially locate the 
;        objects for decomposition.
;
; PROCEDURES USED:
;        This combines most of the decomposition and focus routines in the
;        shapelets code release.
;
; NOTES:
;        The first of the two flags for each object in a shapelet catalogue describes
;        its segmentation into a small postage stamp. The second describes its
;        decomposition into shapelets. Their precise meanings are given below, but see
;        the IDL headers in shapelets_image2pstamp.pro and shapelets_focus.pro
;        for further information.
;
;        MEANING OF FIRST FLAG:
;        0: OK
;        1: did not normalise noise locally (is object filling postage stamp?)
;        2: nearby object (beware overlapping isophotes!)
;        3: both of above
;        4: near edge or a saturated star
;        5: HUGE object (SExtractor gaves whole image occasionally)
;        NB the flag can only take one of these values.
;           If two conditions are duplicated, the higher number takes priority.
;
;        MEANINGS OF SECOND FLAG (from shapelets_focus.pro):
;        0: OK
;        1: has bounced into theta_min or max wall at some point
;        2: nmax_max reached - shapelets incompletely represent object
;        3: chi2>1 but converged (in n_max search) by flatness limit
;        4: did not converge
;        5: centroid off edge
;        6: fatal error
;        NB the flag can only take one of these values.
;           If two conditions are duplicated, the higher number takes priority.
;
; MODIFICATION HISTORY:
;        Jul 04 - index, im_in and scat_in keywords added by AR
;        Jul 04 - too_big paremeter added by RM
;        Jun 04 - Long/integer fixed to allow images with >32768 objects by RM
;                 note: will run into problems with >32768 objects
;                       if the (single precision) Sextractor segmentation map is used
;        May 04 - Sky background subtraction enabled by RM
;        May 04 - Catalogues written to disc as IDL save files by RM
;        Apr 04 - noise_map and seg_map keywords added by Alexandre Refregier
;        Oct 03 - Bug fix on centroiding by RM
;        Sep 03 - Converted for use on non-UNIX platforms by RM
;        Mar 03 - Significantly tidied up by RM
;        Aug 01 - Written by Richard Massey
;-

pro shex, filename_input, INDEX=index,                             $
          IMAGE=image, SEXCAT=sexcat,                              $
          PSF=psf, SKY=sky, TOO_BIG=too_big, SG_CUT=sg_cut,        $
          N_MIN=n_min, N_MAX=n_max,                                $
          NOISE_MAP=noise_map, SEG_MAP=seg_map, CIRCULAR=circular, $
          PLOTIT=plotit, SILENT=silent, VERBOSE=verbose,           $
          SHAPECAT=shapecat,IM_IN=im_in,SCAT_IN=scat_in

COMPILE_OPT idl2
ON_ERROR,2

; Initialise default parameters
if not keyword_set(verbose) then verbose=0B         ; How much output to screen?
if keyword_set(silent) then begin & plotit=0 & verbose=0 & endif
if not keyword_set(n_min) then n_min=2              ; Allowed range of n_max
if not keyword_set(n_max) then n_max=15 else n_max = n_max < 43
if not keyword_set(pixel_scale) then pixel_scale=0. ; Size of pixels in arcseconds
filename=filename_input                             ; Avoid chaging input file name variable
filepath=shapelets_paths(1,/silent)                 ; Automatically prepend data directory opn file names
if not keyword_set(sg_cut) then sg_cut=0.           ; Star/galaxy classification cut
if n_elements(too_big) eq 0 then too_big=100        ; Skip objects bigger than this radius [pixels]
if too_big eq 0 then too_big = 1e10                 ;
case sign(sg_cut) of                                ; Write initial message to screen
  0: message,"Decomposing all objects in "+filename+".fits",/info,noprint=silent
  1: message,"Decomposing galaxies in "+filename+".fits",/info,noprint=silent
 -1: message,"Decomposing stars in "+filename+".fits",/info,noprint=silent
endcase


; Read in SExtractor catalogue and images
if not keyword_set(sexcat) then $
  shapelets_read_sexcat,sexcat,filename,silent=silent
if not keyword_set(image) then $
  shapelets_read_image,image,filename,silent=silent


; Subtract sky background level
if keyword_set(sky) then begin
  if sky ge 3 then begin
    background=2.5*median(image.image)-1.5*mean(image.image)
    message,"Subtracting global background level of "+strtrim(string(background),2),$
            /info,noprint=silent
    image.image=image.image-background
    sky=sky-3
  endif
endif


; Select and reorder the objects as requested
if not keyword_set(index) then begin
  n       = long(n_elements(sexcat.id)) ; Number of objects
  ;reorder = reverse(sort(sexcat.flux)) ; Look at pretty objects first...
  ;reorder = sort(sexcat.flux)          ; ...or low S/N ones first
  reorder = indgen(n,/long)             ; ...or in SExtractor order
  ;reorder = reverse(indgen(n,/long))   ; ...or in reverse SExtractor order
endif else begin
  n=long(n_elements(index))
  reorder=long(index)
endelse


; Determine seeing, and alter the filename if we are deconvolving from a PSF
if n_elements(psf) gt 0 then begin
  seeing=sqrt(shapelets_rsquared(psf))
  filename=filename+'_deconv'
endif else begin
  if ( max(sexcat.class) gt 0.5 ) then $
       seeing=median(sexcat.fwhm[where(sexcat.class gt 0.5)]) $
  else seeing=median(sexcat.fwhm)
endelse


; Prepare empty arrays to contain output catalogue
shapelets_make_nvec,n_max,n1,n2,n_coeffs
x=[[0.],[0.]] 
beta=0.
n_max_cat=0
coeffs=fltarr(1,n_coeffs)
coeffs_error=fltarr(1,n_coeffs)
flag=[[0.],[0.]]
chisq=0.
sexid=0
sexfwhm=0.
sexflux=0.
sexmag=0.
sexclass=0.
sexa=0.
sexb=0.
sextheta=0.
sexe1=0.
sexe2=0.
sexx=0.
sexy=0.
sexarea=0
objx_min=0
objx_max=0
objy_min=0
objy_max=0


; Loop over all objects in SExtractor catalogue, attempting to decompose them
; into shapelets
done =0L
tried=0L
for i=0L,n-1L do begin


  ; Skip stars/galaxies as required
  if verbose ge 1 then print
  if (sg_cut gt 0 and sexcat.class[reorder[i]] gt sg_cut) then begin
    message,'Skipping star #'+strtrim(reorder[i],1)+', '+$
      strtrim(i+1,1)+'/'+strtrim(n,1),/info,noprint=silent
    continue
  endif
  if (sg_cut lt 0 and sexcat.class[reorder[i]] lt abs(sg_cut)) then begin
    message,'Skipping galaxy #'+strtrim(reorder[i],1)+', '+$
      strtrim(i+1,1)+'/'+strtrim(n,1),/info,noprint=silent
    continue
  endif


  ; Skip really big objects to avoid running out of memory and crashing!
  if (sexcat.a[reorder[i]] gt too_big) then begin
    message,'Skipping blood great big object #'+strtrim(reorder[i],1)+', '+$
      strtrim(i+1,1)+'/'+strtrim(n,1),/info,noprint=silent
    continue
  endif
  
  
  ; Write progress report to screen
  message,'Decomposing object #'+strtrim(reorder[i],1)+', '+$
    strtrim(i+1,1)+'/'+strtrim(n,1)+$
    ' at pixel ('+strtrim(round(sexcat.x[reorder[i],0]),1)+ $
    ','+strtrim(round(sexcat.x[reorder[i],1]),1)+')' $
    ,/info,noprint=silent
 
 
  ; Extract postage stamp from larger image
  pstamp=shapelets_sexcat2pstamp(image,sexcat,reorder[i],silent=(1-verbose)>0,$
                                 seg_map=seg_map,noise_map=noise_map,/circular)


  ; If anything has gone wrong so far, skip object
  if pstamp.flag ge 4 then begin
    message,'Skipping object with postage stamp flag='+(strtrim(pstamp.flag,2)),/info,noprint=silent
    continue ; Skip objects near edge of image, etc.
  endif
  tried=tried+1L

  ; Optimise ("focus") shapelet decomposition parameters n_max, beta and x_c:
  shapelets_focus,pstamp,focus,decomp,recomp,verbose=(verbose-1)>0,silent=silent,$
    nmax_ran=[n_min,n_max],psf=psf,th_min_geom=0.2,nmax_g=2>n_min,sky=sky
  x0=focus.x0+[pstamp.x,pstamp.y]-[pstamp.xo,pstamp.yo]


  ; If anything has gone wrong so far, skip object
  if focus.flag ge 4 then begin
    message,'Skipping object with focus flag='+(strtrim(focus.flag,2)),/info,noprint=silent
    continue ; Skip objects whose decomposition did not converge
  endif
  if shapelets_flux(decomp) le 0 then begin
    message,'Skipping object with negative flux',/info,noprint=silent
    continue
  endif
  done=done+1L


  ; Pad coefficient array with zeros to match storage array
  temp_coeffs=decomp.coeffs
  temp_coeffs_error=decomp.coeffs_error
  while (n_elements(temp_coeffs) lt n_coeffs) do begin
    temp_coeffs=[temp_coeffs,0.]
    temp_coeffs_error=[temp_coeffs_error,0.]
  endwhile
  
  
  ; Append individual coefficient array to storage array
  message,'Adding object to catalogue',/info,noprint=(1-verbose)>0
  coeffs=[coeffs,transpose(temp_coeffs)]
  coeffs_error=[coeffs_error,transpose(temp_coeffs_error)]
  x=[x,transpose(x0)]
  beta=[beta,decomp.beta]
  n_max_cat=[n_max_cat,fix(decomp.n_max)]
  flag=[flag,[[pstamp.flag],[focus.flag]]]
  chisq=[chisq,decomp.chisq[1]]
  sexid=[sexid,sexcat.id[reorder[i]]]
  sexfwhm=[sexfwhm,sexcat.fwhm[reorder[i]]]
  sexflux=[sexflux,sexcat.flux[reorder[i]]]
  sexmag=[sexmag,sexcat.mag[reorder[i]]]
  sexclass=[sexclass,sexcat.class[reorder[i]]]
  sexa=[sexa,sexcat.a[reorder[i]]]
  sexb=[sexb,sexcat.b[reorder[i]]]
  sextheta=[sextheta,sexcat.theta[reorder[i]]]
  sexe1=[sexe1,sexcat.e1[reorder[i]]]
  sexe2=[sexe2,sexcat.e2[reorder[i]]]
  sexx=[sexx,sexcat.x[reorder[i],0]]
  sexy=[sexy,sexcat.x[reorder[i],1]]
  sexarea=[sexarea,sexcat.area[reorder[i]]]
  objx_min=[objx_min,pstamp.im_ran[0]]
  objx_max=[objx_max,pstamp.im_ran[1]]
  objy_min=[objy_min,pstamp.im_ran[2]]
  objy_max=[objy_max,pstamp.im_ran[3]]
     
  ; Plot to screen if required
  if keyword_set(plotit) then begin
     plt_objs,pstamp,decomp,image_in=im_in,scat_in=scat_in,focus=focus
;    plt_focus,pstamp,focus,decomp,image_in=im_in
;    plt_pstamp,pstamp,/mask
     read,junk
  endif
 
endfor

   
; Write catalogue to disc
if not keyword_set(silent) then print
if done ne 0 then begin
  shapecat={name:filename,                    $
            type:"shapecat",                  $
            n:done,                           $
            maxn_max:n_max,                   $
            polar:0B,                         $
            seeing:seeing,                    $
            x:x[1:*,*],                       $
            beta:beta[1:*],                   $
            n_max:n_max_cat[1:*],             $
            n_coeffs:n_coeffs,                $
            coeffs:coeffs[1:*,*],             $
            coeffs_error:coeffs_error[1:*,*], $
            flag:flag[1:*,*],                 $
            chisq:chisq[1:*],                 $
            sextractor:1B,                    $
            sexid:sexid[1:*],                 $
            sexfwhm:sexfwhm[1:*],             $
            sexflux:sexflux[1:*],             $
            sexmag:sexmag[1:*],               $
            sexclass:sexclass[1:*],           $
            sexa:sexa[1:*],                   $
            sexb:sexb[1:*],                   $
            sextheta:sextheta[1:*],           $
            sexe1:sexe1[1:*],                 $
            sexe2:sexe2[1:*],                 $
            sexx:sexx[1:*],                   $
            sexy:sexy[1:*],                   $
            sexarea:sexarea[1:*],             $
            objx_min:fix(objx_min[1:*]),      $
            objx_max:fix(objx_max[1:*]),      $
            objy_min:fix(objy_min[1:*]),      $
            objy_max:fix(objy_max[1:*]),      $
            morphology:0B,                    $
            shear:0B}
  shapelets_write_shapecat, shapecat, filename


  ; Write final message to screen proclaiming success!
  if not keyword_set(silent) then begin
    message,'Finished '+filename+'.shape',/info
    print,"Success rate: "+strtrim(string(n),2)+" objects found by SExtractor"
    print,"              "+strtrim(string(tried),2)+" decompositions attempted "+$
          "("+strmid(strtrim(string(100.*tried/n),2),0,5)+"%)"
    print,"              "+strtrim(string(done),2)+" shapelet models "+$
          "("+strmid(strtrim(string(100.*done/tried),2),0,5)+"%)"
    print
  endif

endif else begin
  message,"WARNING: No objects successfully decomposed into shapelets!",/info  
endelse

end
